<?php
/**
 * Member Chimp Drip Content Table List
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

/**
 * MCDC_Admin_Dripping_Table_List class.
 */
class MCDC_Admin_Dripping_Table_List extends WP_List_Table {

	/**
	 * Initialize the table list.
	 */
	public function __construct() {
		parent::__construct(
			array(
				'singular' => 'item',
				'plural'   => 'items',
				'ajax'     => false,
			)
		);
	}

	/**
	 * No items found text.
	 */
	public function no_items() {
		esc_html_e( 'No items found.', 'memberchimp' );
	}

	/**
	 * Get list columns.
	 */
	public function get_columns() {
		return array(
			'cb'            => '<input type="checkbox" />',
			'content_type'  => esc_html__( 'Type', 'memberchimp' ),
			'title'         => esc_html__( 'Content', 'memberchimp' ),
			'delay_access'	=> esc_html__( 'Delay access for / until', 'memberchimp' ),
			'role'			=> esc_html__( 'Role', 'memberchimp' ),
		);
	}

	/**
	 * Column cb.
	 */
	public function column_cb( $item ) {
		return sprintf( '<input type="checkbox" name="item[]" value="%1$s" />', $item['item_id'] );
	}

	/**
	 * Return content_type column.
	 */
	public function column_content_type( $item ) {
		$types = MCDC_Admin_Dripping::get_content_types();
		return '<span class="mcp-tag">' . esc_html( $types[ $item['content_type'] ] ) . '</span>';
	}

	/**
	 * Return title column.
	 */
	public function column_title( $item ) {
		$item_id      = $item['item_id'];
		$term_id 	  = $item['term_id'];
		$content_type = $item['content_type'];
		$url    	  = admin_url( 'admin.php?page=mcp-settings&tab=addons&section=drip&edit-rule=' . $item_id );

		// Get the content title.
		$title = MCDC_Admin_Dripping::get_content_title( $term_id, $content_type );

		// Check if current user can edit other users or if it's the same user.
		$can_edit = current_user_can( 'manage_memberchimp' );

		$output = '<strong>';
		if ( $can_edit ) {
			$output .= '<a href="' . esc_url( $url ) . '" class="row-title">';
		}
		$output .= $title ? $title : esc_html__( 'Unknown', 'memberchimp' );
		if ( $can_edit ) {
			$output .= '</a>';
		}
		$output .= '</strong>';

		// Get actions.
		$actions = array(
			'id' => sprintf( esc_html__( 'ID: %d', 'memberchimp' ), $item['item_id'] ),
		);

		if ( $can_edit ) {
			$actions['edit']  = '<a href="' . esc_url( $url ) . '">' . esc_html__( 'View/Edit', 'memberchimp' ) . '</a>';
			$actions['trash'] = '<a class="submitdelete" aria-label="' . esc_attr__( 'Delete Rule', 'memberchimp' ) . '" href="' . esc_url(
				wp_nonce_url(
					add_query_arg(
						array(
							'delete-rule' => $item['item_id'],
						), admin_url( 'admin.php?page=mcp-settings&tab=addons&section=drip' )
					), 'delete'
				)
			) . '">' . esc_html__( 'Delete', 'memberchimp' ) . '</a>';
		}

		$row_actions = array();

		foreach ( $actions as $action => $link ) {
			$row_actions[] = '<span class="' . esc_attr( $action ) . '">' . $link . '</span>';
		}

		$output .= '<div class="row-actions">' . implode( ' | ', $row_actions ) . '</div>';

		return $output;
	}

	/**
	 * Return delay_access column.
	 */
	public function column_delay_access( $item ) {
		$type  		= $item['type'];
		$number 	= $item['number'];
		$timeframe 	= $item['timeframe'];
		$from_date  = $item['from_date'];

		if ( $type == 'drip' ) {
			if ( $number <= 0 ) {
				return esc_html__( 'Immediately', 'memberchimp' );
			} elseif ( $timeframe == 'days' ) {
				return sprintf( _n( '%d Day', '%d Days', $number, 'memberchimp' ), $number );
			} elseif ( $timeframe == 'weeks' ) {
				return sprintf( _n( '%d Week', '%d Weeks', $number, 'memberchimp' ), $number );
			} elseif ( $timeframe == 'months' ) {
				return sprintf( _n( '%d Month', '%d Months', $number, 'memberchimp' ), $number );
			}
		}

		if ( $type == 'date' ) {
			return date_i18n( mcp_date_format(), strtotime( $from_date ) );
		}
	}

	/**
	 * Return role column.
	 */
	public function column_role( $item ) {
		$roles = memberchimp_get_roles();
		if ( isset( $roles[ $item[ 'role' ] ] ) ) {
			return esc_html( $roles[ $item[ 'role' ] ] );
		} else {
			return esc_html__( 'Registered members', 'memberchimp' );
		}
	}

	/**
	 * Get bulk actions.
	 */
	protected function get_bulk_actions() {
		if ( ! current_user_can( 'manage_memberchimp' ) ) {
			return array();
		}

		return array(
			'delete' => esc_html__( 'Delete', 'memberchimp' ),
		);
	}

	/**
	 * Search box.
	 */
	public function search_box( $text, $input_id ) {
		return;
	}

	/**
	 * Prepare table list items.
	 */
	public function prepare_items() {
		global $wpdb;

		$per_page     = $this->get_items_per_page( 'memberchimp_items_per_page' );
		$current_page = $this->get_pagenum();

		if ( 1 < $current_page ) {
			$offset = $per_page * ( $current_page - 1 );
		} else {
			$offset = 0;
		}

		$search = '';

		// Get the items.
		$items = $wpdb->get_results(
			"SELECT item_id, content_type, term_id, type, number, timeframe, from_date, role FROM {$wpdb->prefix}memberchimp_drip_content WHERE 1 = 1 {$search}" .
			$wpdb->prepare( 'ORDER BY item_id DESC LIMIT %d OFFSET %d;', $per_page, $offset ), ARRAY_A
		);

		$count = $wpdb->get_var( "SELECT COUNT(item_id) FROM {$wpdb->prefix}memberchimp_drip_content WHERE 1 = 1 {$search};" );

		$this->items = $items;

		// Set the pagination.
		$this->set_pagination_args(
			array(
				'total_items' => $count,
				'per_page'    => $per_page,
				'total_pages' => ceil( $count / $per_page ),
			)
		);
	}

}