<?php
/**
 * Content Dripping.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCDC_Content_Dripping class.
 */
class MCDC_Content_Dripping {

	/**
	 * Init.
	 */
	public static function init() {
		add_action( 'template_redirect',	array( __CLASS__, 'drip_content' ), 1 );
		add_filter( 'the_content', 			array( __CLASS__, 'drip_content_filter' ), 1 );
		add_action( 'set_user_role', 		array( __CLASS__, 'role_changed' ), 99, 3 );
	}

	/**
	 * Catch any content dripping rules.
	 */
	public static function drip_content() {
		global $post;

	}

	/**
	 * Filter for the post content if it should not be read.
	 */
	public static function drip_content_filter( $content ) {
		global $post;

		if ( isset( $post->ID ) ) {
			$post_id = absint( $post->ID );
			$drip 	 = self::get_drip( $post_id, is_page() ? 'pages' : 'posts' );
			if ( $drip ) {
				if ( is_wp_error( $can_access = self::user_can_access( $drip ) ) ) {
					$content = $can_access->get_error_message();
				}
			}
			// Check if the category is drip content.
			$cats = wp_get_post_categories( $post_id );
			if ( ! empty( $cats ) ) {
				foreach( $cats as $cat ) {
					$drip 		= self::get_drip( $cat, 'categories' );
					$can_access = self::user_can_access( $drip );
					if ( $drip ) {
						if ( is_wp_error( $can_access ) ) {
							$content = $can_access->get_error_message();
							break;
						}
					}
				}
			}
		}

		return $content;
	}

	/**
	 * Get the drip content rule for an item.
	 */
	public static function get_drip( $term_id, $type = null ) {
		global $wpdb;

		$item = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT *
				FROM {$wpdb->prefix}memberchimp_drip_content
				WHERE term_id = %d AND content_type = %s",
				$term_id,
				$type
			), ARRAY_A
		);

		if ( $item ) {
			return $item;
		} else {
			return null;
		}
	}

	/**
	 * Check if user can access the content.
	 */
	public static function user_can_access( $drip ) {
		if ( ! is_user_logged_in() ) {
			return new WP_Error( 'registration_delay', mcp_get_template_html( 'drip-content/members-only.php', array( 'drip' => $drip ) ) );
		}

		$user    = mcp_get_user( get_current_user_id() );
		$is_role = $drip['role'] && in_array( $drip['role'], array_keys( memberchimp_get_roles() ) );

		// Check the user's role.
		if ( $is_role ) {
			if ( $user->get_role() !== $drip['role'] ) {
				return new WP_Error( 'unallowed_role', mcp_get_template_html( 'drip-content/unallowed-role.php', array( 'role' => $drip['role'], 'user_role' => $user->get_role(), 'user' => $user, 'drip' => $drip ) ) );
			}
		}

		// Delay for amount of time.
		if ( $drip['type'] === 'drip' ) {
			// Immediately available.
			if ( ! $drip['number'] ) {
				return true;
			}

			$registered = $user->get( 'user_registered' );
			$delay 		= ' + ' . absint( $drip['number'] ) . ' ' . esc_attr( $drip['timeframe'] );
			$published  = date( 'Y-m-d H:i:s', strtotime( $registered . $delay ) );

			// The user has not been registered for long enough.
			if ( strtotime( $published ) > time() ) {
				if ( $is_role ) {
					return new WP_Error( 'membership_delay', mcp_get_template_html( 'drip-content/membership-delay.php', array( 'published' => $published, 'registered' => $registered, 'user' => $user, 'drip' => $drip, 'role' => $drip['role'] ) ) );
				} else {
					return new WP_Error( 'registration_delay', mcp_get_template_html( 'drip-content/registration-delay.php', array( 'published' => $published, 'registered' => $registered, 'user' => $user, 'drip' => $drip ) ) );
				}
			}
		}

		// Date-specific drip content.
		if ( $drip['type'] === 'date' ) {
			if ( ! preg_match( '/^[0-9]{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/', $drip['from_date'] ) ) {
				return true;
			}

			// Open this content if the following date is reached.
			$date 		  = $drip['from_date'];
			$current_date = date( 'Y-m-d' );
			
			if ( strtotime( $date ) > strtotime( $current_date ) ) {
				return new WP_Error( 'content_delay', mcp_get_template_html( 'drip-content/content-delay.php', array( 'date' => $date, 'user' => $user, 'drip' => $drip ) ) );
			}
		}

		return true;
	}

	/**
	 * Triggered when user role is changed.
	 */
	public static function role_changed( $user_id, $new_role, $old_roles ) {
		$user = mcp_get_user( $user_id );

		// When the user role is changed?
		$user->set_role_time( $new_role );
	}

}