<?php
/**
 * Front-end Functions.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * This filter add tabs to user profiles for following/followers.
 */
add_filter( 'mcp_get_profile_menu', 'mcp_followers_add_profile_menu', 20 );
function mcp_followers_add_profile_menu( $tabs ) {

	// Check if we should display the followers count in profile nav.
	if ( 'yes' === get_option( 'memberchimp_follower_count_in_menu' ) ) {

		$tabs[ 'following' ]	= array(
			'label'		=> esc_html__( 'Following', 'memberchimp' ),
			'count'		=> true,
			'count_cb'	=> 'mcp_followers_get_following_count',
			'icon'      => 'user-check',
		);

		$tabs[ 'followers' ]	= array(
			'label'		=> esc_html__( 'Followers', 'memberchimp' ),
			'count'		=> true,
			'count_cb'	=> 'mcp_followers_get_followers_count',
			'icon'      => 'users',
		);
	}

	return $tabs;
}

/**
 * Get Following count.
 */
function mcp_followers_get_following_count( $user_id = 0 ) {
	if ( ! $user_id ) {
		$user_id = get_current_user_id();
	}

	$count = get_user_meta( $user_id, '_mcp_following_count', true );

	return number_format_i18n( absint( $count ) );
}

/**
 * Get Followers count.
 */
function mcp_followers_get_followers_count( $user_id = 0 ) {
	if ( ! $user_id ) {
		$user_id = get_current_user_id();
	}

	$count = get_user_meta( $user_id, '_mcp_followers_count', true );

	return number_format_i18n( absint( $count ) );
}

/**
 * Add the profile followers counts to meta.
 */
add_action( 'memberchimp_profile_meta_counts', 'mcp_followers_add_counts', 20 );
function mcp_followers_add_counts( $meta_counts ) {

	$meta_counts[] = '<a href="' . esc_url( mcp_get_profile_endpoint_url( 'following' ) ) . '" class="mcp-following-count" data-user="' . mcp_get_user_id() . '">' . sprintf( __( '<strong>%s</strong> Following', 'memberchimp' ), mcp_followers_get_following_count( mcp_get_user_id() ) ) . '</a>';

	$meta_counts[] = '<a href="' . esc_url( mcp_get_profile_endpoint_url( 'followers' ) ) . '" class="mcp-followers-count" data-user="' . mcp_get_user_id() . '">' . sprintf( __( '<strong>%s</strong> Followers', 'memberchimp' ), mcp_followers_get_followers_count( mcp_get_user_id() ) ) . '</a>';

	return $meta_counts;
}

/**
 * Display the follow button in profile nav.
 */
add_action( 'memberchimp_extend_profile_buttons', 'mcp_followers_add_button', 20 );
function mcp_followers_add_button() {
	if ( ! is_user_logged_in() || is_mcp_my_profile( mcp_get_user_id() ) ) {
		return;
	}

	$state = mcp_followers_has_followed( get_current_user_id(), mcp_get_user_id() );

	mcp_get_template(
		'followers/follow-button.php',
		array(
			'state' => $state
		)
	);
}

/**
 * Add the security nonce to follow/unfollow user.
 */
add_filter( 'mcp_ajax_nonces', 'mcp_followers_ajax_nonces' );
function mcp_followers_ajax_nonces( $nonces ) {
	$nonces[ 'follow_user' ] = wp_create_nonce( 'memberchimp-follow-user' );
	return $nonces;
}

/**
 * Registers frontend scripts.
 */
add_action( 'wp_enqueue_scripts', 'mcp_followers_register_js', 20 );
function mcp_followers_register_js() {
	wp_register_script( 'mcp-followers', mcp_followers()->plugin_url() . '/assets/js/mcp-followers.js', array( 'jquery' ), mcp()->version, true );
}

/**
 * Add javascript dependancy to memberchimp.
 */
add_filter( 'memberchimp_javascript_deps', 'mcp_followers_add_js_deps' );
function mcp_followers_add_js_deps( $deps ) {
	array_push( $deps, 'mcp-followers' );

	return $deps;
}

/**
 * Basically checks if first user has followed second user.
 */
function mcp_followers_has_followed( $user_a, $user_b ) {
	$user_a = absint( $user_a );
	$user_b = absint( $user_b );

	if ( ! $user_a || ! $user_b ) {
		return false;
	}

	$get_following = get_user_meta( $user_a, '_mcp_following', true );

	if ( in_array( $user_b, ( array ) $get_following ) ) {
		return true;
	}

	return false;
}

/**
 * Add a follow.
 */
function mcp_followers_follow( $user_a, $user_b ) {
	$user_a_following = get_user_meta( $user_a, '_mcp_following', true );
	$user_b_followers = get_user_meta( $user_b, '_mcp_followers', true );

	$a_count = get_user_meta( $user_a, '_mcp_following_count', true );
	$b_count = get_user_meta( $user_b, '_mcp_followers_count', true );

	if ( ! $user_b_followers ) {
		$user_b_followers = array();
	}
	if ( ! $user_a_following ) {
		$user_a_following = array();
	}

	$user_a_following[] = $user_b;
	$user_b_followers[] = $user_a;

	update_user_meta( $user_a, '_mcp_following', $user_a_following );
	update_user_meta( $user_b, '_mcp_followers', $user_b_followers );

	update_user_meta( $user_a, '_mcp_following_count', absint( $a_count ) + 1 );
	update_user_meta( $user_b, '_mcp_followers_count', absint( $b_count ) + 1 );

	$result = array(
		'user_id' => $user_b,
		'current' => $user_a,
		'user_a'  => number_format_i18n( absint( $a_count ) + 1 ),
		'user_b'  => number_format_i18n( absint( $b_count ) + 1 ),
		'state'   => 'followed',
	);

	return $result;
}

/**
 * Unfollow a user.
 */
function mcp_followers_unfollow( $user_a, $user_b ) {
	$user_a_following = get_user_meta( $user_a, '_mcp_following', true );
	$user_b_followers = get_user_meta( $user_b, '_mcp_followers', true );

	$a_count = get_user_meta( $user_a, '_mcp_following_count', true );
	$b_count = get_user_meta( $user_b, '_mcp_followers_count', true );

	if ( ( $key = array_search( $user_b, $user_a_following ) ) !== false ) {
		unset( $user_a_following[$key] );
	}
	if ( ( $key = array_search( $user_a, $user_b_followers ) ) !== false ) {
		unset( $user_b_followers[$key] );
	}

	if ( empty( $user_a_following ) ) {
		$user_a_following = null;
	}

	if ( empty( $user_b_followers ) ) {
		$user_b_followers = null;
	}

	update_user_meta( $user_a, '_mcp_following', $user_a_following );
	update_user_meta( $user_b, '_mcp_followers', $user_b_followers );

	update_user_meta( $user_a, '_mcp_following_count', absint( $a_count ) - 1 );
	update_user_meta( $user_b, '_mcp_followers_count', absint( $b_count ) - 1 );

	$result = array(
		'user_id' => $user_b,
		'current' => $user_a,
		'user_a'  => number_format_i18n( absint( $a_count ) - 1 ),
		'user_b'  => number_format_i18n( absint( $b_count ) - 1 ),
		'state'   => 'unfollowed',
	);

	return $result;
}

/**
 * Display the followed users list in profile tab.
 */
add_action( 'memberchimp_profile_following_endpoint', 'memberchimp_profile_following_endpoint' );
function memberchimp_profile_following_endpoint() {	
	$user_id = mcp_get_active_profile_id();
	$users   = get_user_meta( $user_id, '_mcp_following', true );

	if ( $users ) {
		mcp_get_template( 'followers/following.php', array( 'users' => array_reverse( $users ) ) );
	} else {
		mcp_get_template( 'followers/no-following.php' );
	}
}

/**
 * Display the followers list in profile tab.
 */
add_action( 'memberchimp_profile_followers_endpoint', 'memberchimp_profile_followers_endpoint' );
function memberchimp_profile_followers_endpoint() {	
	$user_id = mcp_get_active_profile_id();
	$users   = get_user_meta( $user_id, '_mcp_followers', true );

	if ( $users ) {
		mcp_get_template( 'followers/followers.php', array( 'users' => array_reverse( $users ) ) );
	} else {
		mcp_get_template( 'followers/no-followers.php' );
	}
}

/**
 * Add the follow button in memberlist.
 */
add_action( 'memberchimp_before_usercard_actions', 'mcp_followers_add_memberlist_button', 20 );
function mcp_followers_add_memberlist_button() {
	mcp_followers_add_button();
}

/**
 * Fix the count when no user was found in user's following/followers.
 */
function mcp_followers_fix_count( $user_id = 0, $type = 'following' ) {
	$user  = mcp_get_active_profile_id();
	$users = get_user_meta( $user, '_mcp_' . $type, true );
	$count = get_user_meta( $user, '_mcp_' . $type . '_count', true );

	if ( ( $key = array_search( $user_id, $users ) ) !== false ) {
		unset( $users[$key] );
	}

	if ( empty( $users ) ) {
		$users = null;
	}

	update_user_meta( $user, '_mcp_' . $type, $users );
	update_user_meta( $user, '_mcp_' . $type . '_count', absint( $count ) - 1 );
}