<?php
/**
 * Social Networks class.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCSO_Networks class.
 */
class MCSO_Networks {

	public static $networks;

	/**
	 * Hook in tabs.
	 */
	public static function init() {
		self::$networks = apply_filters( 'mcp_social_networks_array', array( 'facebook', 'twitter' ) );

		foreach( self::$networks as $network ) {
			include_once 'networks/class-mcso-' . esc_attr( $network ) . '.php';
		}
	}

	/**
	 * Get an authorized social request.
	 */
	public static function authorize() {
		if ( ! isset( $_REQUEST['auth'] ) || ! in_array( $_REQUEST['auth'], self::$networks ) ) {
			return;
		}

		$network = esc_attr( $_REQUEST['auth'] );
		$class   = 'MCSO_' . ucfirst( $network );
		$class::authorize();
	}

	/**
	 * Get enabled networks.
	 */
	public static function get_enabled() {
		foreach( self::$networks as $key => $network ) {
			if ( ! self::is_enabled( $network ) ) {
				unset( self::$networks[ $key ] );
			}
		}
		return self::$networks;
	}

	/**
	 * Checks if a social network is enabled.
	 */
	public static function is_enabled( $network ) {
		$class = 'MCSO_' . ucfirst( $network );
		return $class::is_enabled() ? true : false;
	}

	/**
	 * Process a social connect. The workflow.
	 */
	public static function process( $data ) {
		global $the_form, $the_role;

		extract( $data );

		// See if user previously connected via this social network.
		$user_found = mcp_social_user_found( $data );

		if ( is_numeric( $user_found ) ) {
			self::connect_existing_user( $user_found, $data );
		} else if ( $user_id = email_exists( $email ) ) {
			self::connect_existing_user( $user_id, $data );
		} else {
			self::new_social_user( $data );
		}
	}

	/**
	 * Connect existing social user.
	 */
	public static function connect_existing_user( $user_id, $data ) {
		global $the_form, $the_role;

		extract( $data );

		// Email was found, log in user.
		$user = mcp_get_user( $user_id );

		$user = apply_filters( 'wp_authenticate_user', $user, '' );
		$user = apply_filters( 'memberchimp_valid_login_credentials', $user );

		// When the user should not login.
		if ( is_wp_error( $user ) ) {
			wp_safe_redirect( add_query_arg( 'saved', $user->get_error_code(), untrailingslashit( wp_get_referer() ) ) );
			exit;
		} else {
			$user->log_in();
		}

		update_user_meta( $user->ID, '_mcp_' . $provider . '_id', mcp_clean( $uid ) );

		// Handle redirect.
		if ( is_mcp_account_page() ) {
			$redirect = mcp_get_page_permalink( 'account' );
		} else {
			$redirect = mcp_get_login_redirect( $user );
		}
		exit( wp_safe_redirect( esc_url( $redirect ) ) );
	}

	/**
	 * Creates a new user.
	 */
	public static function new_social_user( $data ) {
		global $wpdb, $the_form, $the_role;

		extract( $data );

		// Identify user's login.
		if ( isset( $username ) ) {
			$login = $username;
		} else {
			$login = $uid;
		}

		$count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(ID) FROM {$wpdb->users} where user_login = %s", $login ) );
		if ( $count ) {
			$login = $login . $count;
		}

		// Creating the user.
		$userdata = array(
			'user_login'  => $login,
			'user_email'  => $email,
			'role'		  => get_option( 'memberchimp_social_default_role' ),
		);

		// Add extra captured user data.
		if ( isset( $name ) ) {
			$userdata['display_name'] = $name;
		}

		// Create the user.
		$user = mcp_create_user( $userdata );

		// This role maybe needs admin approval.
		$role_id = $user->get_role_id();

		if ( ! $role_id ) {
			$the_role = null;
		}

		$the_role = new MCP_Role( $role_id );
		if ( $user->needs_manual_review() ) {
			$user->create_pending_user();
		}

		$user = apply_filters( 'wp_authenticate_user', $user, '' );
		$user = apply_filters( 'memberchimp_valid_login_credentials', $user );

		// When the user should not login.
		if ( is_wp_error( $user ) ) {
			wp_safe_redirect( add_query_arg( 'saved', $user->get_error_code(), untrailingslashit( wp_get_referer() ) ) );
			exit;
		} else {
			$user->log_in();
		}

		/*
		 * Imports the user avatar when possible.
		 */
		if ( ! empty( $avatar ) ) {
			mcp_social_import_avatar( $user->ID, $avatar );
		}

		update_user_meta( $user->ID, '_mcp_' . $provider . '_id', mcp_clean( $uid ) );

		// Handle redirect.
		if ( is_mcp_account_page() ) {
			$redirect = mcp_get_page_permalink( 'account' );
		} else {
			$redirect = mcp_get_register_redirect( $user );
		}
		exit( wp_safe_redirect( esc_url( $redirect ) ) );
	}

}