<?php
/**
 * Setup Wizard
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCP_Admin_Setup_Wizard class.
 */
class MCP_Admin_Setup_Wizard {

	/**
	 * Current step
	 */
	private $step = '';

	/**
	 * Steps for the setup wizard
	 */
	private $steps = array();

	/**
	 * Hook in tabs.
	 */
	public function __construct() {
		if ( apply_filters( 'memberchimp_enable_setup_wizard', true ) && current_user_can( 'manage_memberchimp' ) ) {
			add_action( 'admin_menu', array( $this, 'admin_menus' ) );
			add_action( 'admin_init', array( $this, 'setup_wizard' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		}
	}

	/**
	 * Add admin menus/screens.
	 */
	public function admin_menus() {
		add_dashboard_page( '', '', 'manage_options', 'mcp-setup', '' );
	}

	/**
	 * Register/enqueue scripts and styles for the Setup Wizard.
	 */
	public function enqueue_scripts() {
		wp_enqueue_style( 'mcp-setup', mcp()->plugin_url() . '/assets/css/mcp-setup.css', array( 'dashicons', 'install' ), MCP_VERSION );

		// Add RTL support.
		wp_style_add_data( 'mcp-setup', 'rtl', 'replace' );

		wp_register_script( 'mcp-setup', mcp()->plugin_url() . '/assets/js/admin/mcp-setup.js', array( 'jquery', 'jquery-tiptip', 'jquery-toggles' ), MCP_VERSION );
		wp_localize_script(
			'mcp-setup',
			'mcp_setup_params',
			array(
				'current_step'	=> isset( $this->steps[ $this->step ] ) ? $this->step : false,
			)
		);
	}

	/**
	 * Show the setup wizard.
	 */
	public function setup_wizard() {
		if ( empty( $_GET['page'] ) || 'mcp-setup' !== $_GET['page'] ) {
			return;
		}

		$default_steps = array(
			'initial' 		=> array(
				'name'    	=> esc_html__( 'Initial Setup', 'memberchimp' ),
				'handler' 	=> array( $this, 'mcp_setup_initial_save' ),
			),
			'addons'     	=> array(
				'name'    	=> esc_html__( 'Install Features', 'memberchimp' ),
				'handler' 	=> array( $this, 'mcp_setup_addons_save' ),
			),
			'ready'			=> array(
				'name'		=> esc_html__( 'Ready!', 'memberchimp' ),
				'handler' 	=> '',
			),
		);

		$this->steps = apply_filters( 'memberchimp_setup_wizard_steps', $default_steps );
		$this->step  = isset( $_GET['step'] ) ? sanitize_key( $_GET['step'] ) : current( array_keys( $this->steps ) );

		if ( ! empty( $_POST['save_step'] ) && isset( $this->steps[ $this->step ]['handler'] ) ) {
			call_user_func( $this->setup_wizard_save(), $this );
		}

		ob_start();
		$this->setup_wizard_header();
		$this->setup_wizard_steps();
		$this->setup_wizard_content();
		$this->setup_wizard_footer();
		exit;
	}

	/**
	 * Get the URL for the next step's screen.
	 */
	public function get_next_step_link( $step = '' ) {
		if ( ! $step ) {
			$step = $this->step;
		}

		$keys = array_keys( $this->steps );
		if ( end( $keys ) === $step ) {
			return admin_url();
		}

		$step_index = array_search( $step, $keys, true );
		if ( false === $step_index ) {
			return '';
		}

		return add_query_arg( 'step', $keys[ $step_index + 1 ], remove_query_arg( 'activate_error' ) );
	}

	/**
	 * Setup Wizard Header.
	 */
	public function setup_wizard_header() {
		set_current_screen();
		?>
		<!DOCTYPE html>
		<html <?php language_attributes(); ?>>
		<head>
			<meta name="viewport" content="width=device-width" />
			<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
			<title><?php esc_html_e( 'Member Chimp &rsaquo; Setup Wizard', 'memberchimp' ); ?></title>
			<?php do_action( 'admin_enqueue_scripts' ); ?>
			<?php wp_print_scripts( 'mcp-setup' ); ?>
			<?php do_action( 'admin_print_styles' ); ?>
			<?php do_action( 'admin_head' ); ?>
		</head>
		<body class="mcp-setup wp-core-ui">
			<h1 id="mcp-logo"><a href="https://memberchimp.com" title="<?php esc_html_e( 'Member Chimp', 'memberchimp' ); ?>"><img src="<?php echo mcp()->plugin_url(); ?>/assets/images/memberchimp-logo.png" alt="<?php esc_html_e( 'Member Chimp Logo', 'memberchimp' ); ?>" /></a></h1>
		<?php
	}

	/**
	 * Setup Wizard Footer.
	 */
	public function setup_wizard_footer() {
		?>
			<?php if ( 'initial' === $this->step ) : ?>
				<a class="mcp-setup-footer-links" href="<?php echo esc_url( admin_url() ); ?>"><?php esc_html_e( 'Not right now', 'memberchimp' ); ?></a>
			<?php elseif ( 'initial' !== $this->step && 'ready' !== $this->step ) : ?>
				<a class="mcp-setup-footer-links" href="<?php echo esc_url( $this->get_next_step_link() ); ?>"><?php esc_html_e( 'Skip this step', 'memberchimp' ); ?></a>
			<?php endif; ?>
			</body>
		</html>
		<?php
	}

	/**
	 * Output the steps.
	 */
	public function setup_wizard_steps() {
		$output_steps      = $this->steps;
		?>
		<ol class="mcp-setup-steps">
			<?php
			foreach ( $output_steps as $step_key => $step ) {
				$is_completed = array_search( $this->step, array_keys( $this->steps ), true ) > array_search( $step_key, array_keys( $this->steps ), true );

				if ( $step_key === $this->step ) {
					?>
					<li class="active"><?php echo esc_html( $step['name'] ); ?></li>
					<?php
				} elseif ( $is_completed ) {
					?>
					<li class="done">
						<a href="<?php echo esc_url( add_query_arg( 'step', $step_key, remove_query_arg( 'activate_error' ) ) ); ?>"><?php echo esc_html( $step['name'] ); ?></a>
					</li>
					<?php
				} else {
					?>
					<li><?php echo esc_html( $step['name'] ); ?></li>
					<?php
				}
			}
			?>
		</ol>
		<?php
	}

	/**
	 * Output the content for the current step.
	 */
	public function setup_wizard_content() {
		global $thepostid;
		$thepostid = -1;
		echo '<div class="mcp-setup-content">';
		if ( ! @include dirname( __FILE__ ) . '/views/html-setup-' . $this->step . '.php' ) {
			esc_html_e( 'Failed to load setup wizard.', 'memberchimp' );
		}
		echo '</div>';
	}

	/**
	 * Save progress.
	 */
	public function setup_wizard_save() {
		check_admin_referer( 'mcp-setup' );
		call_user_func( $this->steps[ $this->step ]['handler'], $this );
		wp_safe_redirect( esc_url_raw( $this->get_next_step_link() ) );
		exit;
	}

	/**
	 * Save initial step.
	 */
	public function mcp_setup_initial_save() {

		// Create base.
		$defaults = array(
			'_forms',
			'_fields',
			'_roles',
			'_memberlists',
		);

		foreach( $defaults as $items ) {
			call_user_func( 'memberchimp_create_default' . $items );
		}

		// Create pages.
		$pages[ 'profile' ] = array(
			'name'    => _x( 'profile', 'Page slug', 'memberchimp' ),
			'title'   => _x( 'Profile', 'Page title', 'memberchimp' ),
			'content' => '[memberchimp_profile]',
		);

		$pages[ 'account' ] = array(
			'name'    => _x( 'account', 'Page slug', 'memberchimp' ),
			'title'   => _x( 'Account', 'Page title', 'memberchimp' ),
			'content' => '[memberchimp_account]',
		);

		$pages[ 'register' ] = array(
			'name'    => _x( 'register', 'Page slug', 'memberchimp' ),
			'title'   => _x( 'Register', 'Page title', 'memberchimp' ),
			'content' => '[memberchimp_register]',
		);

		$pages[ 'login' ] = array(
			'name'    => _x( 'login', 'Page slug', 'memberchimp' ),
			'title'   => _x( 'Login', 'Page title', 'memberchimp' ),
			'content' => '[memberchimp_login]',
		);

		$pages[ 'lostpassword' ] = array(
			'name'    => _x( 'lost-password', 'Page slug', 'memberchimp' ),
			'title'   => _x( 'Lost password', 'Page title', 'memberchimp' ),
			'content' => '[memberchimp_lostpassword]',
		);

		$pages[ 'memberlist' ] = array(
			'name'    => _x( 'members', 'Page slug', 'memberchimp' ),
			'title'   => _x( 'Members', 'Page title', 'memberchimp' ),
			'content' => '[memberchimp_memberlist]',
		);

		foreach ( (array) $pages as $key => $page ) {
			mcp_create_page( esc_sql( $page['name'] ), 'memberchimp_' . $key . '_page_id', $page['title'], $page['content'], '' );
		}

		// Done with page setup. Save permalinks.
		update_option( 'memberchimp_queue_flush_rewrite_rules', 'yes' );
		mcp()->query->init_query_vars();
		mcp()->query->add_endpoints();

	}

	/**
	 * Save addons step.
	 */
	public function mcp_setup_addons_save() {
		// Save or unsave activated addons.
		$activated = array();
		$addons = mcp()->addons->get_addons();
		foreach( $addons as $addon => $array ) {
			if ( isset( $_POST[ mcp_sanitize_title( $addon ) ] ) ) {
				$activated[] = $addon;
				update_option( 'memberchimp_' . mcp_sanitize_title( $addon ), 'yes' );
			} else {
				update_option( 'memberchimp_' . mcp_sanitize_title( $addon ), 'no' );
			}
		}

		delete_option( 'memberchimp_enabled_addons' );
		update_option( 'memberchimp_enabled_addons', $activated );
	}

}

new MCP_Admin_Setup_Wizard();