<?php
/**
 * Users Admin
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCP_Admin_Users class.
 */
class MCP_Admin_Users {

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_filter( 'user_row_actions', array( $this, 'user_row_actions' ), 10, 2 );

		add_filter( 'manage_users_columns', array( $this, 'add_custom_columns' ), 99 );
		add_action( 'manage_users_custom_column', array( $this, 'view_custom_columns' ), 99, 3 );
		add_filter( 'manage_users_sortable_columns', array( $this, 'define_sortable_columns' ), 99 );

		add_action( 'pre_user_query', array( $this, 'order_users_by_date' ), 99 );

		// Add status filter.
		add_action( 'restrict_manage_users', array( $this, 'show_status_filter' ) );
		add_filter( 'pre_get_users', array( $this, 'filter_by_status' ) );

		// Action Hooks.
		add_action( 'delete_user', array( $this, 'delete_user_data' ) );
	}

	/**
	 * User row actions.
	 */
	public function user_row_actions( $actions, $user ) {
		$actions[ 'view' ] = '<a href="' . memberchimp_get_profile_url( $user->user_login ) . '" aria-label="' . esc_html__( 'View user profile', 'memberchimp' ) . '">' . esc_html__( 'View', 'memberchimp' ) . '</a>';

		return $actions;
	}

	/**
	 * Add custom columns.
	 */
	public function add_custom_columns( $columns ) {
		$checkbox = $columns[ 'cb' ];
		unset( $columns[ 'cb' ] );

		$columns[ 'mcp_date' ] 	  = _x( 'Registered', 'user', 'memberchimp' );
		$columns[ 'mcp_actions' ] = '';

		return array_merge( array( 'cb' => $checkbox, 'mcp_status' => '<span class="mcp_status tips" data-tip="' . esc_attr( 'Status', 'memberchimp' ) . '">' . mcp_svg_icon( 'more-horizontal' ) . '</span>' ), $columns );
	}

	/**
	 * Control the output of custom columns.
	 */
	public function view_custom_columns( $value, $column_name, $user_id ) {

		// Status
		if ( 'mcp_status' == $column_name ) {
			if ( is_mcp_unconfirmed_email_user( $user_id ) ) {
				$html = '<span class="status-email tips" data-tip="' . esc_attr__( 'Unconfirmed', 'memberchimp' ) . '">' . mcp_svg_icon( 'mail' ) . '</span>';
			} elseif ( is_mcp_rejected_user( $user_id ) ) {
				$html = '<span class="status-rejected tips" data-tip="' . esc_attr__( 'Rejected', 'memberchimp' ) . '">' . mcp_svg_icon( 'x' ) . '</span>';
			} elseif ( is_mcp_pending_user( $user_id ) ) {
				$html = '<span class="status-pending tips" data-tip="' . esc_attr__( 'Pending', 'memberchimp' ) . '">' . mcp_svg_icon( 'more-horizontal' ) . '</span>';
			} else {
				$html = '<span class="status-active tips" data-tip="' . esc_attr__( 'Active', 'memberchimp' ) . '">' . mcp_svg_icon( 'check' ) . '</span>';
			}

			return $html;
		}

		// Date registered.
		if ( 'mcp_date' == $column_name ) {
			$user = get_userdata( $user_id );

			$registered = strtotime( get_date_from_gmt( $user->user_registered ) );

			$date = sprintf( esc_html__( '%1$s at %2$s', 'memberchimp' ), date_i18n( mcp_date_format(), $registered ), date_i18n( mcp_time_format(), $registered ) );
	
			return $date;
		}

		// Actions.
		if ( 'mcp_actions' == $column_name ) {
			$output  = null;
			$actions = $this->get_user_actions( $user_id );

			if ( empty( $actions ) ) {
				return;
			}

			foreach( $actions as $action ) {
				$output .= $action;
			}

			$output .= '<span class="spinner">' . esc_html__( 'Processing...', 'memberchimp' ) . '</span>';
			$output .= '<span class="updater">' . esc_html__( 'Action completed', 'memberchimp' ) . '</span>';

			return $output;
		}

		return $value;
	}

	/**
	 * Recent users to appear first.
	 */
	public function order_users_by_date( $query ) {
		global $pagenow;

		if ( ! is_admin() || 'users.php' !== $pagenow || isset( $_GET[ 'orderby' ] ) ) {
			return;
		}

		$query->query_orderby = 'ORDER BY user_registered DESC';	
	}

	/**
	 * Makes the column sortable
	 */
	public function define_sortable_columns( $columns ) {
		$custom = array(
			'mcp_date'	=> 'registered',
		);
		return wp_parse_args( $custom, $columns );
	}

	/**
	 * Get the user actions buttons.
	 */
	public function get_user_actions( $user_id = 0 ) {
		$actions = array();

		if ( user_can( $user_id, 'manage_memberchimp' ) ) {
			return;
		}

		if ( ! is_mcp_unconfirmed_email_user( $user_id ) ) {
			$actions[] = '<a href="' . wp_nonce_url( admin_url( 'admin-ajax.php?action=memberchimp_confirm_email&id=' . $user_id ), 'mcp-confirm-email' ) . '" class="button mcp_ajax mcp-top-tips" data-tip="' . esc_attr( 'Request email confirmation', 'memberchimp' ) . '">' . mcp_svg_icon( 'mail' ) . '</a>';
		}

		if ( is_mcp_pending_user( $user_id ) ) {
			if ( ! is_mcp_rejected_user( $user_id ) ) {
				$actions[] = '<a href="' . wp_nonce_url( admin_url( 'admin-ajax.php?action=memberchimp_approve_user&id=' . $user_id ), 'mcp-approve-user' ) . '" class="button mcp_ajax mcp-top-tips" data-tip="' . esc_attr( 'Approve user', 'memberchimp' ) . '">' . mcp_svg_icon( 'check' ) . '</a>';
				$actions[] = '<a href="' . wp_nonce_url( admin_url( 'admin-ajax.php?action=memberchimp_reject_user&id=' . $user_id ), 'mcp-reject-user' ) . '" class="button mcp_ajax mcp-top-tips" data-tip="' . esc_attr( 'Reject user', 'memberchimp' ) .'">' . mcp_svg_icon( 'x' ) . '</a>';
			} else {
				$actions[] = '<a href="' . wp_nonce_url( admin_url( 'admin-ajax.php?action=memberchimp_reinstate_user&id=' . $user_id ), 'mcp-reinstate-user' ) . '" class="button mcp_ajax mcp-top-tips" data-tip="' . esc_attr( 'Reinstate user', 'memberchimp' ) .'">' . mcp_svg_icon( 'refresh-cw' ) . '</a>';
			}
		} else {
			$actions[] = '<a href="' . wp_nonce_url( admin_url( 'admin-ajax.php?action=memberchimp_set_pending&id=' . $user_id ), 'mcp-set-pending' ) . '" class="button mcp_ajax mcp-top-tips" data-tip="'. esc_attr( 'Set as pending', 'memberchimp' ) . '">' . mcp_svg_icon( 'clock' ) . '</a>';
		}

		$actions[] = '<a href="' . memberchimp_get_profile_url( $user_id ) . '" class="button mcp-top-tips" data-tip="' . esc_attr( 'View profile', 'memberchimp' ) .'">' . mcp_svg_icon( 'eye' ) . '</a>';
		$actions[] = '<a href="' . memberchimp_get_edit_user_link( $user_id ) . '" class="button mcp-top-tips" data-tip="' . esc_attr( 'Edit profile', 'memberchimp' ) . '">' . mcp_svg_icon( 'edit-2' ) . '</a>';

		return apply_filters( 'memberchimp_users_admin_actions', $actions, $user_id );
	}

	/**
	 * Show status filter.
	 */
	public function show_status_filter( $which ) {
		global $wp_list_table;
		if ( $which != 'top' ) {
			return;
		}

		$selected = ! empty( $_GET[ 'mcp_status' ] ) ? esc_attr( $_GET[ 'mcp_status' ] ) : '';

		if ( ! $wp_list_table->has_items() ) {
			$select = '<select name="mcp_status" class="mcp-select mcp-select-top nomargin">';
		} else {
			$select = '<select name="mcp_status" class="mcp-select mcp-select-top">';
		}

		$select .= '<option value="">' . esc_html__( 'Filter by status...', 'memberchimp' ). '</option>';

		foreach( mcp_get_user_statuses() as $key => $value ) {
			$select .= '<option value="' . esc_attr( $key ) . '" ' . selected( $selected, $key, false ) . '>' . esc_html( $value ) . '</option>';
		}

		$select .= '</select>';

		echo $select;

		submit_button( esc_html__( 'Filter', 'memberchimp' ), null, $which, false );
	}

	/**
	 * Filter users by a specific status.
	 */
	public function filter_by_status( $query ) {
		global $pagenow;
		if ( is_admin() && 'users.php' == $pagenow && ! empty( $_GET[ 'mcp_status' ] ) ) {
			$status = esc_attr( $_GET[ 'mcp_status' ] );
			switch( $status ) {
				case 'pending' :
					$meta_query = array(
						array(
							'key'     => '_mcp_pending',
							'value'   => 1,
							'compare' => '=',
						),
						array(
							'key'     => '_mcp_rejected',
							'compare' => 'NOT EXISTS',
						),
					);
				break;
				case 'rejected' :
					$meta_query = array(
						array(
							'key'     => '_mcp_rejected',
							'value'   => 1,
							'compare' => '=',
						),
					);
				break;
				case 'unconfirmed' :
					$meta_query = array(
						array(
							'key'     => '_mcp_unconfirmed_email',
							'value'   => 1,
							'compare' => '=',
						),
					);
				break;
				case 'approved' :
					$meta_query = array(
						array(
							'key'     => '_mcp_pending',
							'compare' => 'NOT EXISTS',
						),
						array(
							'key'     => '_mcp_rejected',
							'compare' => 'NOT EXISTS',
						),
						array(
							'key'     => '_mcp_unconfirmed_email',
							'compare' => 'NOT EXISTS',
						),
					);
				break;
			}
			if ( isset( $meta_query ) ) {
				$query->set( 'meta_query', $meta_query );
			}
		}
	}

	/**
	 * Deletes user data when a user is deleted.
	 */
	function delete_user_data( $user_id ) {
		include_once MCP_ABSPATH . 'includes/mcp-user-hooks.php';
		mcp_delete_user_files( $user_id );
	}

}

new MCP_Admin_Users();