<?php
/**
 * Admin.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCP_Admin class.
 */
class MCP_Admin {

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_action( 'init', array( $this, 'includes' ) );
		add_action( 'current_screen', array( $this, 'conditional_includes' ) );
		add_action( 'admin_init', array( $this, 'buffer' ), 1 );
		add_action( 'admin_init', array( $this, 'preview_emails' ) );
		add_action( 'admin_init', array( $this, 'prevent_admin_access' ) );
		add_action( 'admin_init', array( $this, 'admin_redirects' ) );
		add_action( 'admin_footer', 'mcp_print_js', 25 );
		add_action( 'admin_footer_text', array( $this, 'admin_footer_text' ), 1 );
	}

	/**
	 * Include any classes we need within admin.
	 */
	public function includes() {
		include_once dirname( __FILE__ ) . '/mcp-admin-functions.php';
		include_once dirname( __FILE__ ) . '/mcp-meta-box-functions.php';
		include_once dirname( __FILE__ ) . '/class-mcp-admin-post-types.php';
		include_once dirname( __FILE__ ) . '/class-mcp-admin-menus.php';
		include_once dirname( __FILE__ ) . '/class-mcp-admin-notices.php';
		include_once dirname( __FILE__ ) . '/class-mcp-admin-assets.php';
		include_once dirname( __FILE__ ) . '/class-mcp-admin-api-keys.php';

		// Setup/welcome
		if ( ! empty( $_GET['page'] ) ) {
			switch ( $_GET['page'] ) {
				case 'mcp-setup':
					include_once dirname( __FILE__ ) . '/class-mcp-admin-setup-wizard.php';
				break;
			}
		}
	}

	/**
	 * Include admin files conditionally.
	 */
	public function conditional_includes() {
		$screen = get_current_screen();

		if ( ! $screen ) {
			return;
		}

		switch ( $screen->id ) {
			case 'dashboard' :
				include 'class-mcp-admin-dashboard.php';
				break;
			case 'users':
			case 'user':
			case 'profile':
			case 'user-edit':
				include dirname( __FILE__ ) . '/class-mcp-admin-users.php';
				break;
		}
	}

	/**
	 * Output buffering allows admin screens to make redirects later on.
	 */
	public function buffer() {
		ob_start();
	}

	/**
	 * Preview email template.
	 */
	public function preview_emails() {

		if ( isset( $_GET['preview_memberchimp_mail'] ) ) {
			if ( ! ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'preview-mail' ) ) ) {
				die( esc_html__( 'Security check.', 'memberchimp' ) );
			}

			// load the mailer class.
			$mailer = mcp()->mailer();

			// get the preview email subject.
			$email_heading = esc_html__( 'HTML email preview', 'memberchimp' );

			// get the preview email content.
			ob_start();
			include 'views/html-email-template-preview.php';
			$message = ob_get_clean();

			// create a new email.
			$email = new MCP_Email();

			// wrap the content with the email template and then add styles.
			$message = apply_filters( 'memberchimp_mail_content', $email->style_inline( $mailer->wrap_message( $email_heading, $message ) ) );

			echo $message;
			exit;
		}
	}

	/**
	 * Check if admin access should be prevented.
	 */
	public function prevent_admin_access() {
		global $pagenow;
		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return;
		}

		if ( is_admin() && is_user_logged_in() && ! current_user_can( 'mcp_view_wpadmin' ) ) {
			if ( ! current_user_can( 'manage_options' ) ) {
				exit( wp_safe_redirect( home_url() ) );
			}
		}
	}

	/**
	 * Handle redirects to setup/welcome page after install and updates.
	 *
	 * For setup wizard, transient must be present, the user must have access rights, and we must ignore the network/bulk plugin updaters.
	 */
	public function admin_redirects() {

		// Setup wizard redirect.
		if ( get_transient( '_mcp_activation_redirect' ) && apply_filters( 'memberchimp_enable_setup_wizard', true ) ) {
			$do_redirect  = true;
			$current_page = isset( $_GET['page'] ) ? mcp_clean( wp_unslash( $_GET['page'] ) ) : false;

			// On these pages, or during these events, postpone the redirect.
			if ( wp_doing_ajax() || is_network_admin() || ! current_user_can( 'manage_memberchimp' ) ) {
				$do_redirect = false;
			}

			// On these pages, or during these events, disable the redirect.
			if ( 'mcp-setup' === $current_page || ! MCP_Admin_Notices::has_notice( 'install' ) || apply_filters( 'memberchimp_prevent_automatic_wizard_redirect', false ) || isset( $_GET['activate-multi'] ) ) {
				delete_transient( '_mcp_activation_redirect' );
				$do_redirect = false;
			}

			if ( $do_redirect ) {
				delete_transient( '_mcp_activation_redirect' );
				wp_safe_redirect( admin_url( 'index.php?page=mcp-setup' ) );
				exit;
			}
		}
	}

	/**
	 * Change the admin footer text on Member Chimp admin pages.
	 */
	public function admin_footer_text( $footer_text ) {
		if ( ! current_user_can( 'manage_memberchimp' ) || ! function_exists( 'mcp_get_screen_ids' ) ) {
			return $footer_text;
		}

		$current_screen  = get_current_screen();
		$mcp_pages        = mcp_get_screen_ids();

		// Set only Member Chimp pages.
		$mcp_pages = array_diff( $mcp_pages, array( 'profile', 'user-edit' ) );

		// Check to make sure we're on a admin page.
		if ( isset( $current_screen->id ) && apply_filters( 'memberchimp_display_admin_footer_text', in_array( $current_screen->id, $mcp_pages, true ) ) ) {
			// Change the footer text.
			if ( ! get_option( 'memberchimp_admin_footer_text_rated' ) ) {
				$footer_text = sprintf( wp_kses_post( __( 'Thank you for using %s.', 'memberchimp' ) ), '<a href="https://memberchimp.com" target="_blank">' . esc_html__( 'Member Chimp', 'memberchimp' ) . '</a>' );
				mcp_enqueue_js(
					"jQuery( 'a.mcp-rating-link' ).click( function() {
						jQuery.post( '" . mcp()->ajax_url() . "', { action: 'memberchimp_rated' } );
						jQuery( this ).parent().text( jQuery( this ).data( 'rated' ) );
					});"
				);
			} else {
				$footer_text = sprintf( wp_kses_post( __( 'Thank you for using %s.', 'memberchimp' ) ), '<a href="https://memberchimp.com" target="_blank">' . esc_html__( 'Member Chimp', 'memberchimp' ) . '</a>' );
			}
		}

		return $footer_text;
	}

}

return new MCP_Admin();