<?php
/**
 * AJAX Events.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCP_AJAX class.
 */
class MCP_AJAX {

	/**
	 * Hook in ajax handlers.
	 */
	public static function init() {
		self::add_ajax_events();
	}

	/**
	 * Hook in methods - uses WordPress ajax handlers (admin-ajax).
	 */
	public static function add_ajax_events() {
		// memberchimp_EVENT => nopriv.
		$ajax_events = array(
			'check_confirmation_code'	=> true,
			'send_form'					=> true,
			'cancel_pending_email'		=> false,
			'resend_confirmation'		=> false,
			'save_avatar'				=> false,
			'save_cover'				=> false,
			'remove_avatar'				=> false,
			'remove_cover'				=> false,
			'unblock_user'				=> false,
			'block_user'				=> false,
			'delete_user'				=> false,
			'save_form'					=> false,
			'add_field'					=> false,
			'create_forms'				=> false,
			'create_fields'				=> false,
			'create_roles'				=> false,
			'create_memberlists'		=> false,
			'duplicate_form'			=> false,
			'duplicate_field'			=> false,
			'duplicate_role'			=> false,
			'duplicate_memberlist'		=> false,
			'confirm_email'				=> false,
			'set_pending'				=> false,
			'approve_user'				=> false,
			'reject_user'				=> false,
			'reinstate_user'			=> false,
			'rated'						=> false,
			'update_api_key'			=> false,
			'json_search_users'			=> false,
			'json_search_terms'			=> false,
		);

		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_memberchimp_' . $ajax_event, array( __CLASS__, $ajax_event ) );

			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_memberchimp_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			}
		}
	}

	/**
	 * Could not perform an ajax action.
	 */
	public static function could_not_perform_action() {
		$array = array(
			'message'	=> apply_filters( 'mcp_could_not_perform_action', esc_html__( 'Could not perform this action.', 'memberchimp' ) ),
		);

		return $array;
	}

	/**
	 * Checks a confirmation code and completes user registration.
	 */
	public static function check_confirmation_code() {
		check_ajax_referer( 'memberchimp-confirmation-code', 'security' );

		$user_id 		   = isset( $_POST[ 'user_id' ] ) ? absint( $_POST[ 'user_id' ] ) : '';
		$confirmation_code = isset( $_POST[ 'confirmation_code' ] ) ? absint( $_POST[ 'confirmation_code' ] ) : '';

		$user = mcp_get_user( $user_id );

		$result = $user->process_confirmation_code( $confirmation_code );

		if ( ! $result || is_wp_error( $result ) ) {
			wp_send_json( array( 'error' => 'true' ) );
		} else {
			wp_send_json( array( 'redirect' => $result ) );
		}

	}

	/**
	 * Send a form.
	 */
	public static function send_form() {
		global $the_form;

		/**
		 * This will take care of security checks.
		 */
		call_user_func(
			array(
				'MCP_Form_Handler',
				mcp_sanitize_title( wp_unslash( $_REQUEST[ '_endpoint' ] ) )
			)
		);

		$response = array(
			'html' 			=> mcp_print_notices( true ),
			'error_fields' 	=> $the_form->get_error_fields(),
			'js_redirect'	=> $the_form->js_redirect,
		);

		// Maybe clear the data from form after completion.
		if ( isset( $the_form->cleardata ) ) {
			$response[ 'cleardata' ] = 'true';
		}

		wp_send_json( $response );
	}

	/**
	 * Cancel pending email change.
	 */
	public static function cancel_pending_email() {

		check_ajax_referer( 'memberchimp-cancel-email', 'security' );

		if ( ! is_user_logged_in() || ! current_user_can( 'mcp_edit_account' ) ) {
			return;
		}

		mcp_user_cancel_pending_email( get_current_user_id() );

		$response = array(
			'message' => esc_html__( 'Your email change request was cancelled.', 'memberchimp' ),
		);

		wp_send_json( $response );
	}

	/**
	 * Resend confirmation for email change.
	 */
	public static function resend_confirmation() {

		check_ajax_referer( 'memberchimp-resend-confirmation', 'security' );

		if ( ! is_user_logged_in() || ! current_user_can( 'mcp_edit_account' ) ) {
			return;
		}

		$user = mcp_get_user( get_current_user_id() );

		$secret_key = $user->create_email_change_key();

		// Load the mailer class.
		mcp()->mailer();
		do_action( 'memberchimp_email_change_notification', $user, $secret_key );

		$response = array(
			'message' => esc_html__( 'A confirmation email has been sent to you.', 'memberchimp' ),
		);

		wp_send_json( $response );
	}

	/**
	 * Save user avatar.
	 */
	public static function save_avatar() {

		check_ajax_referer( 'memberchimp-save-avatar', 'security' );

		$user_id = isset( $_POST[ 'user_id' ] ) ? absint( $_POST[ 'user_id' ] ) : '';

		if ( ! mcp_user_can_edit_profile( $user_id ) ) {
			wp_die( -1 );
		}

		// Upload the avatar.
		$file_return = mcp_upload_avatar( $_POST );

		// Return errors.
		if ( isset( $file_return[ 'error' ] ) ) {
			wp_send_json( $file_return );
		}

		// Return correct avatar markup when the user avatar is updated.
		$user = get_userdata( $user_id );
		if ( mcp_user_has_no_gravatar( $user_id, $user->user_email ) ) {
			$html = '<div class="memberchimp-profile-photo-add-wrap">
						<a href="' . esc_url( mcp_get_profile_endpoint_url( 'edit', $user->user_login ) ) . '" class="memberchimp-profile-photo-add mcp-dropdown-init" rel="mcp-dropdown-photo">' . mcp_svg_icon( 'camera' ) . '</a>
					</div>';
		} else {
			$html = '<a href="' . memberchimp_get_profile_url( $user->user_login ) . '" class="memberchimp-profile-photo-link">' . get_avatar( $user_id, 200 ) . '</a>';
		}

		$response = array(
			'message'		=> esc_html__( 'Your profile photo was published successfully.', 'memberchimp' ),
			'id'			=> $user_id,
			'file_return'	=> $file_return,
			'js_update'				=> array(
				'parent'			=> '.memberchimp-profile-photo[data-user_id=' . $user_id . ']',
				'child'				=> '.memberchimp-profile-photo-add-wrap, .memberchimp-profile-photo-link',
				'html'				=> $html,
			),
			'toggle'		=> array(
				'.mcp_no_avatar'	=> 'mcp_has_avatar',
			),
		);

		wp_send_json( $response );
	}

	/**
	 * Remove user avatar.
	 */
	public static function remove_avatar() {

		check_ajax_referer( 'memberchimp-remove-avatar', 'security' );

		$user_id = isset( $_POST[ 'user_id' ] ) ? absint( $_POST[ 'user_id' ] ) : '';

		if ( ! mcp_user_can_edit_profile( $user_id ) ) {
			wp_die( -1 );
		}

		// Removes the avatar.
		mcp_delete_user_avatar( $user_id, true );

		// Return correct avatar markup when the user avatar is updated.
		$user = get_userdata( $user_id );
		if ( mcp_user_has_no_gravatar( $user_id, $user->user_email ) ) {
			$html = '<div class="memberchimp-profile-photo-add-wrap">
						<a href="' . esc_url( mcp_get_profile_endpoint_url( 'edit', $user->user_login ) ) . '" class="memberchimp-profile-photo-add mcp-dropdown-init" rel="mcp-dropdown-photo">' . mcp_svg_icon( 'camera' ) . '</a>
					</div>';
		} else {
			$html = '<a href="' . memberchimp_get_profile_url( $user->user_login ) . '" class="memberchimp-profile-photo-link">' . get_avatar( $user_id, 200 ) . '</a>';
		}

		$response = array(
			'message'		=>__( 'Your profile photo was removed.', 'memberchimp' ),
			'id'			=> $user_id,
			'js_update'				=> array(
				'parent'			=> '.memberchimp-profile-photo[data-user_id=' . $user_id . ']',
				'child'				=> '.memberchimp-profile-photo-add-wrap, .memberchimp-profile-photo-link',
				'html'				=> $html,
			),
			'toggle'		=> array(
				'.mcp_has_avatar'	=> 'mcp_no_avatar',
			),
		);

		wp_send_json( $response );
	}

	/**
	 * Save user cover.
	 */
	public static function save_cover() {

		check_ajax_referer( 'memberchimp-save-cover', 'security' );

		$user_id = isset( $_POST[ 'user_id' ] ) ? absint( $_POST[ 'user_id' ] ) : '';

		if ( ! mcp_user_can_edit_profile( $user_id ) ) {
			wp_die( -1 );
		}

		// Upload the cover.
		$file_return = mcp_upload_cover( $_POST );

		$html = '<div class="memberchimp-profile-coverbg" style="background-image: url( ' . mcp_get_user_cover_url( $user_id ) . ' );"></div>';

		$response = array(
			'message'		=> esc_html__( 'Your header photo was published successfully.', 'memberchimp' ),
			'id'			=> $user_id,
			'file_return'	=> $file_return,
			'js_update'				=> array(
				'parent'			=> '.memberchimp-profile-cover',
				'child'				=> '.memberchimp-profile-coverbg',
				'html'				=> $html,
			),
			'toggle'		=> array(
				'.mcp_no_cover'		=> 'mcp_has_cover',
			),
		);

		wp_send_json( $response );
	}

	/**
	 * Remove user cover.
	 */
	public static function remove_cover() {

		check_ajax_referer( 'memberchimp-remove-cover', 'security' );

		$user_id = isset( $_POST[ 'user_id' ] ) ? absint( $_POST[ 'user_id' ] ) : '';

		if ( ! mcp_user_can_edit_profile( $user_id ) ) {
			wp_die( -1 );
		}

		// Removes the cover.
		mcp_delete_user_cover( $user_id, true );

		$html = '<div class="memberchimp-profile-coverbg" style="background-image: url( ' . mcp_get_user_cover_url( $user_id ) . ' );"></div>';

		$response = array(
			'message'		=> esc_html__( 'Your profile header was removed.', 'memberchimp' ),
			'id'			=> $user_id,
			'js_update'				=> array(
				'parent'			=> '.memberchimp-profile-cover',
				'child'				=> '.memberchimp-profile-coverbg',
				'html'				=> $html,
			),
			'toggle'		=> array(
				'.mcp_has_cover'	=> 'mcp_no_cover',
			),
		);

		wp_send_json( $response );
	}

	/**
	 * Unblock a user.
	 */
	public static function unblock_user() {

		check_ajax_referer( 'memberchimp-unblock-user', 'security' );

		if ( ! $user = mcp_user_can_block( $_POST[ 'user_id' ] ) ) {
			wp_send_json( self::could_not_perform_action() );
		}

		// Unblock user.
		if ( mcp_unblock_user( $user ) > 0 ) {
			$message = sprintf( esc_html__( '@%s is no longer blocked.', 'memberchimp' ), $user->user_login );
		} else {
			$message = sprintf( esc_html__( '@%s could not be unblocked.', 'memberchimp' ), $user->user_login );
		}

		$response = array(
			'id'			=> $user->ID,
			'message'		=> $message,
			'js_update'				=> array(
				'parent'			=> '#mcp-user-' . $user->ID,
			),
			'toggle'		=> array(
				'.mcp_blocked'		=> 'mcp_unblocked',
			),
			'keep_alive'	=> true,
		);

		wp_send_json( $response );
	}

	/**
	 * Block a user.
	 */
	public static function block_user() {

		check_ajax_referer( 'memberchimp-block-user', 'security' );

		if ( ! $user = mcp_user_can_block( $_POST[ 'user_id' ] ) ) {
			wp_send_json( self::could_not_perform_action() );
		}

		// Block user.
		if ( mcp_block_user( $user ) > 0 ) {
			$message = sprintf( esc_html__( '@%s is now blocked.', 'memberchimp' ), $user->user_login );
		} else {
			$message = sprintf( esc_html__( '@%s could not be blocked.', 'memberchimp' ), $user->user_login );
		}

		$response = array(
			'id'			=> $user->ID,
			'message'		=> $message,
			'js_update'				=> array(
				'parent'			=> '#mcp-user-' . $user->ID,
			),
			'toggle'		=> array(
				'.mcp_unblocked'	=> 'mcp_blocked',
			),
		);

		wp_send_json( $response );
	}

	/**
	 * Delete a user.
	 */
	public static function delete_user() {

		check_ajax_referer( 'memberchimp-delete-user', 'security' );

		// Make sure that user can delete other users.
		if ( ! current_user_can( 'mcp_delete_users' ) ) {
			wp_send_json( self::could_not_perform_action() );
		}

		$user_id = isset( $_POST[ 'user_id' ] ) && ! empty( $_POST[ 'user_id' ] ) ? absint( $_POST[ 'user_id' ] ) : 0;

		if ( $user = mcp_delete_user( $_POST[ 'user_id' ] ) ) {
			$response = array(
				'id'		=> $user->ID,
				'message'	=> sprintf( esc_html__( '@%s is now deleted.', 'memberchimp' ), $user->user_login ),
				'js_remove'	=> '#mcp-user-' . $user->ID,
			);

			wp_send_json( $response );
		} else {

			wp_send_json( self::could_not_perform_action() );
		}
	}

	/**
	 * Save a form.
	 */
	public static function save_form() {

		check_ajax_referer( 'memberchimp-save-form', 'security' );

		if ( ! current_user_can( 'edit_mcp_forms' ) ) {
			wp_die( -1 );
		}

		$the_form = new MCP_Form( absint( $_POST['id'] ) );
		$the_form->save( array(
			'rows'	    => isset( $_POST['rows'] ) ? mcp_clean( $_POST['rows'] ) : '', 
			'fields'	=> isset( $_POST['fields'] ) ? mcp_clean( $_POST['fields'] ) : '',
			'row_count'	=> absint( $_POST['row_count'] ),
			'cols'		=> mcp_clean( $_POST['cols'] ),
		) );

		wp_die();
	}

	/**
	 * Add a field.
	 */
	public static function add_field() {

		check_ajax_referer( 'memberchimp-add-field', 'security' );

		if ( ! current_user_can( 'edit_mcp_fields' ) ) {
			wp_die( -1 );
		}

		$errors = array();
		$props = array();

		$the_field = new MCP_Field();

		$props['key']         			= isset( $_POST['key'] ) ? sanitize_title( wp_unslash( $_POST['key'] ) ) : '';

		if ( ! $props['key'] ) {
			$errors['key'] = esc_html__( 'You must provide a unique key for this custom field.', 'memberchimp' );
		}

		if ( $the_field->exists( $props['key'] ) ) {
			$errors['key'] = esc_html__( 'The key provided is already in use. Please write a unique key.', 'memberchimp' );
		}

		// Send errors back to form.
		if ( ! empty( $errors ) ) {
			wp_send_json( array( 'errors' => $errors ) );
		}

		// No errors? Setup all props now.
		$props = mcp_setup_field_props( 'label' );

		// Add the field to database.
		if ( empty( $errors ) ) {
			$the_field->set( 'post_title', $props['label'] );
			$the_field->set( 'post_name', mcp_clean( wp_unslash( $props['key'] ) ) );
			$the_field->set( 'meta_input', $props );
			$the_field->insert();
			$the_field->save( $the_field->meta_input );
		}

		$html = '<a href="#" class="button button-secondary insert_field" ' . mcp_get_data_attributes( $props ) . '>' . esc_html__( $props['label'] ) . '</a>';

		wp_send_json( array( 'data' => $html ) );
	}

	/**
	 * Create default forms.
	 */
	public static function create_forms() {

		check_ajax_referer( 'memberchimp-create-forms', 'security' );

		if ( ! current_user_can( 'publish_mcp_forms' ) ) {
			wp_die( -1 );
		}

		memberchimp_create_default_forms();

		wp_die();
	}

	/**
	 * Create default custom fields.
	 */
	public static function create_fields() {

		check_ajax_referer( 'memberchimp-create-fields', 'security' );

		if ( ! current_user_can( 'publish_mcp_fields' ) ) {
			wp_die( -1 );
		}

		memberchimp_create_default_fields();

		wp_die();
	}

	/**
	 * Create default user roles.
	 */
	public static function create_roles() {

		check_ajax_referer( 'memberchimp-create-roles', 'security' );

		if ( ! current_user_can( 'publish_mcp_roles' ) ) {
			wp_die( -1 );
		}

		memberchimp_create_default_roles();

		wp_send_json( array( 'redirect' => admin_url( 'edit.php?post_type=mcp_role' ) ) );
	}

	/**
	 * Create default member lists.
	 */
	public static function create_memberlists() {

		check_ajax_referer( 'memberchimp-create-memberlists', 'security' );

		if ( ! current_user_can( 'publish_mcp_memberlists' ) ) {
			wp_die( -1 );
		}

		memberchimp_create_default_memberlists();

		wp_die();
	}

	/**
	 * Duplicate a form.
	 */
	public static function duplicate_form() {

		check_ajax_referer( 'duplicate-form', 'security' );

		if ( ! current_user_can( 'publish_mcp_forms' ) ) {
			wp_die( -1 );
		}

		$item = new MCP_Form( absint( $_POST[ 'id' ] ) );
		$item->duplicate();

		wp_die();
	}

	/**
	 * Duplicate a custom field.
	 */
	public static function duplicate_field() {

		check_ajax_referer( 'duplicate-field', 'security' );

		if ( ! current_user_can( 'publish_mcp_fields' ) ) {
			wp_die( -1 );
		}

		$item = new MCP_Field( absint( $_POST[ 'id' ] ) );
		$item->duplicate();

		wp_die();
	}

	/**
	 * Duplicate a user role.
	 */
	public static function duplicate_role() {

		check_ajax_referer( 'duplicate-role', 'security' );

		if ( ! current_user_can( 'publish_mcp_roles' ) ) {
			wp_die( -1 );
		}

		$item = new MCP_Role( absint( $_POST[ 'id' ] ) );
		$item->duplicate();

		wp_die();
	}

	/**
	 * Duplicate a member list.
	 */
	public static function duplicate_memberlist() {

		check_ajax_referer( 'duplicate-memberlist', 'security' );

		if ( ! current_user_can( 'publish_mcp_memberlists' ) ) {
			wp_die( -1 );
		}

		$item = new MCP_Memberlist( absint( $_POST[ 'id' ] ) );
		$item->duplicate();

		wp_die();
	}

	/**
	 * Initiate email confirmation from admin.
	 */
	public static function confirm_email() {
		if ( ! ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'mcp-confirm-email' ) ) ) {
			die( esc_html__( 'Security check.', 'memberchimp' ) );
		}

		$user_id = absint( $_REQUEST[ 'id' ] );

		if ( ! current_user_can( 'mcp_review_users' ) ) {
			wp_die( -1 );
		}

		$user = mcp_get_user( $user_id );
		$user->create_confirmation_code( true );

		$response = array(
			'updated'	=> esc_html__( 'Email confirmation requested.', 'memberchimp' ),
		);

		wp_send_json( $response );
	}

	/**
	 * Set user as pending.
	 */
	public static function set_pending() {
		if ( ! ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'mcp-set-pending' ) ) ) {
			die( esc_html__( 'Security check.', 'memberchimp' ) );
		}

		$user_id = absint( $_REQUEST[ 'id' ] );

		if ( ! current_user_can( 'mcp_review_users' ) ) {
			wp_die( -1 );
		}

		$user = mcp_get_user( $user_id );
		$user->create_pending_user();

		wp_send_json( array( 'updated' => esc_html__( 'User is now pending.', 'memberchimp' ) ) );
	}

	/**
	 * Approve a user.
	 */
	public static function approve_user() {
		if ( ! ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'mcp-approve-user' ) ) ) {
			die( esc_html__( 'Security check.', 'memberchimp' ) );
		}

		$user_id = absint( $_REQUEST[ 'id' ] );

		if ( ! current_user_can( 'mcp_review_users' ) ) {
			wp_die( -1 );
		}

		$user = mcp_get_user( $user_id );
		$user->approve_pending_user();

		wp_send_json( array( 'updated' => esc_html__( 'User is now approved.', 'memberchimp' ) ) );
	}

	/**
	 * Reject a user.
	 */
	public static function reject_user() {
		if ( ! ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'mcp-reject-user' ) ) ) {
			die( esc_html__( 'Security check.', 'memberchimp' ) );
		}

		$user_id = absint( $_REQUEST[ 'id' ] );

		if ( ! current_user_can( 'mcp_review_users' ) ) {
			wp_die( -1 );
		}

		$user = mcp_get_user( $user_id );
		$user->reject_pending_user();

		wp_send_json( array( 'updated' => esc_html__( 'User is moved to auto deletion queue.', 'memberchimp' ) ) );
	}

	/**
	 * Reinstate a user.
	 */
	public static function reinstate_user() {
		if ( ! ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'mcp-reinstate-user' ) ) ) {
			die( esc_html__( 'Security check.', 'memberchimp' ) );
		}

		$user_id = absint( $_REQUEST[ 'id' ] );

		if ( ! current_user_can( 'mcp_review_users' ) ) {
			wp_die( -1 );
		}

		$user = mcp_get_user( $user_id );
		$user->reinstate();

		wp_send_json( array( 'updated' => esc_html__( 'User is now pending.', 'memberchimp' ) ) );
	}

	/**
	 * Triggered when clicking the rating footer.
	 */
	public static function rated() {
		if ( ! current_user_can( 'manage_memberchimp' ) ) {
			wp_die( -1 );
		}
		update_option( 'memberchimp_admin_footer_text_rated', 1 );
		wp_die();
	}

	/**
	 * Create/Update API key.
	 */
	public static function update_api_key() {
		ob_start();

		global $wpdb;

		check_ajax_referer( 'update-api-key', 'security' );

		if ( ! current_user_can( 'manage_memberchimp' ) ) {
			wp_die( -1 );
		}

		$response = array();

		try {
			if ( empty( $_POST['description'] ) ) {
				throw new Exception( esc_html__( 'Description is missing.', 'memberchimp' ) );
			}
			if ( empty( $_POST['user'] ) ) {
				throw new Exception( esc_html__( 'User is missing.', 'memberchimp' ) );
			}
			if ( empty( $_POST['permissions'] ) ) {
				throw new Exception( esc_html__( 'Permissions is missing.', 'memberchimp' ) );
			}

			$key_id      = isset( $_POST['key_id'] ) ? absint( $_POST['key_id'] ) : 0;
			$description = sanitize_text_field( wp_unslash( $_POST['description'] ) );
			$permissions = ( in_array( wp_unslash( $_POST['permissions'] ), array( 'read', 'write', 'read_write' ), true ) ) ? sanitize_text_field( wp_unslash( $_POST['permissions'] ) ) : 'read';
			$user_id     = absint( $_POST['user'] );

			// Check if current user can edit other users.
			if ( $user_id && ! current_user_can( 'edit_user', $user_id ) ) {
				if ( get_current_user_id() !== $user_id ) {
					throw new Exception( esc_html__( 'You do not have permission to assign API Keys to the selected user.', 'memberchimp' ) );
				}
			}

			if ( 0 < $key_id ) {
				$data = array(
					'user_id'     => $user_id,
					'description' => $description,
					'permissions' => $permissions,
				);

				$wpdb->update(
					$wpdb->prefix . 'memberchimp_api_keys',
					$data,
					array( 'key_id' => $key_id ),
					array(
						'%d',
						'%s',
						'%s',
					),
					array( '%d' )
				);

				$response                    = $data;
				$response['consumer_key']    = '';
				$response['consumer_secret'] = '';
				$response['message']         = esc_html__( 'API Key updated successfully.', 'memberchimp' );
			} else {
				$consumer_key    = 'ck_' . mcp_rand_hash();
				$consumer_secret = 'cs_' . mcp_rand_hash();

				$data = array(
					'user_id'         => $user_id,
					'description'     => $description,
					'permissions'     => $permissions,
					'consumer_key'    => mcp_api_hash( $consumer_key ),
					'consumer_secret' => $consumer_secret,
					'truncated_key'   => substr( $consumer_key, -7 ),
				);

				$wpdb->insert(
					$wpdb->prefix . 'memberchimp_api_keys',
					$data,
					array(
						'%d',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
					)
				);

				$key_id                      = $wpdb->insert_id;
				$response                    = $data;
				$response['consumer_key']    = $consumer_key;
				$response['consumer_secret'] = $consumer_secret;
				$response['message']         = esc_html__( 'API Key generated successfully. Make sure to copy your new keys now as the secret key will be hidden once you leave this page.', 'memberchimp' );
				$response['revoke_url']      = '<a style="color: #a00; text-decoration: none;" href="' . esc_url( wp_nonce_url( add_query_arg( array( 'revoke-key' => $key_id ), admin_url( 'admin.php?page=mcp-settings&tab=advanced&section=keys' ) ), 'revoke' ) ) . '">' . esc_html__( 'Revoke key', 'memberchimp' ) . '</a>';
			}
		} catch ( Exception $e ) {
			wp_send_json_error( array( 'message' => $e->getMessage() ) );
		}

		// wp_send_json_success must be outside the try block not to break phpunit tests.
		wp_send_json_success( $response );
	}

	/**
	 * Search users and get result in JSON format.
	 */
	public static function json_search_users() {

		check_ajax_referer( 'memberchimp-json-search-users', 'security' );

		if ( ! current_user_can( 'manage_memberchimp' ) ) {
			wp_die( -1 );
		}

		$results = array();

		$users = new WP_User_Query( array(
			'search'         => '*' . esc_attr( $_REQUEST[ 'keyword' ] ) . '*',
			'search_columns' => array(
				'user_login',
				'user_nicename',
				'user_email',
				'user_url',
			),
		) );

		$users_found = $users->get_results();

		if ( ! empty( $users_found ) ) {
			foreach( $users_found as $user ) {
				$results[] = array(
					'id'	=> $user->ID,
					'email'	=> $user->user_email,
					'name'	=> $user->display_name,
				);
			}
		}

		wp_send_json( $results );
	}

	/**
	 * Search terms and get result in JSON format.
	 */
	public static function json_search_terms() {

		check_ajax_referer( 'memberchimp-json-search-terms', 'security' );

		if ( ! current_user_can( 'manage_memberchimp' ) ) {
			wp_die( -1 );
		}

		$results = array();

		$type = esc_attr( $_REQUEST[ 'type' ] );

		if ( in_array( $type, array( 'posts', 'pages', 'categories' ) ) ) {
			$data_store = MCP_Data_Store::load( $type );
			$method 	= 'search_' . $type;
			$results 	= $data_store->$method( mcp_clean( esc_attr( $_REQUEST[ 'keyword' ] ) ) );
		}

		wp_send_json( $results );
	}

}

MCP_AJAX::init();