<?php
/**
 * Form Handler.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCP_Form_Handler class.
 */
class MCP_Form_Handler {

	/**
	 * Hook in methods.
	 */
	public static function init() {
		// Detect password reset key and redirect.
		add_action( 'template_redirect', array( __CLASS__, 'redirect_reset_password_link' ) );

		// Detect email change key and redirect.
		add_action( 'template_redirect', array( __CLASS__, 'redirect_email_change_link' ) );

		// Login & Registration.
		add_action( 'template_redirect', array( __CLASS__, 'login' ) );
		add_action( 'template_redirect', array( __CLASS__, 'register' ) );

		// Lost password.
		add_action( 'template_redirect', array( __CLASS__, 'lostpassword' ) );
		add_action( 'template_redirect', array( __CLASS__, 'password_reset' ) );

		// Edit profile.
		add_action( 'template_redirect', array( __CLASS__, 'profile' ) );

		// Account actions.
		add_action( 'template_redirect', array( __CLASS__, 'edit_account' ) );
		add_action( 'template_redirect', array( __CLASS__, 'edit_password' ) );
		add_action( 'template_redirect', array( __CLASS__, 'privacy' ) );

		// Triggered when user profile or account is shown.
		add_action( 'template_redirect', array( __CLASS__, 'profile_redirect' ), 50 );
		add_action( 'template_redirect', array( __CLASS__, 'account_redirect' ), 50 );

		// Trigger a login as another user.
		add_action( 'template_redirect', array( __CLASS__, 'login_as_user' ), 9 );

		// WP admin screen redirects.
		add_action( 'init', array( __CLASS__, 'wp_admin_screen' ) );

		// Hooks for core validation.
		add_action( 'memberchimp_account_validation', array( __CLASS__, 'maybe_validate_username' ), 10 );
		add_action( 'memberchimp_account_validation', array( __CLASS__, 'maybe_validate_email' ), 20 );
		add_action( 'memberchimp_edit_password_validation', array( __CLASS__, 'maybe_validate_password' ), 10 );
	}

	/**
	 * Remove key and user ID from query string, and redirect to account page to show the form.
	 */
	public static function redirect_reset_password_link() {
		if ( isset( $_GET['password_key'] ) && ( isset( $_GET['id'] ) ) ) {
			$user_id = absint( $_GET['id'] );
			$value = sprintf( '%d:%s', $user_id, wp_unslash( $_GET['password_key'] ) );

			MCP_Shortcode_Form_Lostpassword::set_reset_password_cookie( $value );

			wp_safe_redirect( add_query_arg( 'show-reset-form', 'true', mcp_lostpassword_url() ) );
			exit;
		}
	}

	/**
	 * Validate email change tokens from url.
	 */
	public static function redirect_email_change_link() {
		global $wpdb;

		if ( isset( $_GET['email_key'] ) && ( isset( $_GET['id'] ) ) ) {
			$the_user = mcp_get_user( absint( $_GET['id'] ) );
			if ( isset( $the_user->ID ) ) {
				$the_user->process_email_change_key( $_GET['email_key'] );
			}
		}

	}

	/**
	 * Get valid handler.
	 */
	public static function handle( $mode ) {
		if ( ! isset( $_REQUEST['memberchimp-' . $mode . '-nonce'] ) 
			|| ! wp_verify_nonce( $_REQUEST['memberchimp-' . $mode. '-nonce'], 'memberchimp-' . $mode )
		) {
			return false;
		}
		if ( ! empty( $_REQUEST[ '_' . $mode . '_id' ] ) ) {
			return absint( $_REQUEST[ '_' . $mode . '_id' ] );
		}
		return false;
	}

	/**
	 * Login.
	 */
	public static function login() {
		if ( ! $id = self::handle( 'login' ) ) {
			return;
		}
		MCP_Shortcode_Form_Login::save( mcp_get_form( $id ) );
	}

	/**
	 * Register.
	 */
	public static function register() {
		if ( ! $id = self::handle( 'register' ) ) {
			return;
		}
		MCP_Shortcode_Form_Register::save( mcp_get_form( $id ) );
	}

	/**
	 * Lost Password.
	 */
	public static function lostpassword() {
		if ( ! $id = self::handle( 'lostpassword' ) ) {
			return;
		}
		MCP_Shortcode_Form_Lostpassword::save( mcp_get_form( $id ) );
	}

	/**
	 * Password reset.
	 */
	public static function password_reset() {
		if ( ! $id = self::handle( 'password_reset' ) ) {
			return;
		}
		MCP_Shortcode_Form_Lostpassword::process_password_change( mcp_get_form( $id ) );
	}

	/**
	 * Profile - Edit
	 */
	public static function profile() {
		if ( ! $id = self::handle( 'edit-profile' ) ) {
			return;
		}
		MCP_Shortcode_Profile::save( mcp_get_form( $id ) );
	}

	/**
	 * Account - Edit
	 */
	public static function edit_account() {
		if ( ! $id = self::handle( 'edit-account' ) ) {
			return;
		}
		MCP_Shortcode_Form_Account::save( mcp_get_form( $id ) );
	}

	/**
	 * Account - Edit password
	 */
	public static function edit_password() {
		if ( ! $id = self::handle( 'edit-password' ) ) {
			return;
		}
		MCP_Shortcode_Form_Account::save( mcp_get_form( $id ) );
	}

	/**
	 * Account - Privacy
	 */
	public static function privacy() {
		if ( ! $id = self::handle( 'privacy' ) ) {
			return;
		}
		MCP_Shortcode_Form_Account::save( mcp_get_form( $id ) );
	}

	/**
	 * Profile redirect.
	 */
	public static function profile_redirect() {
		global $current_user;

		if ( ! is_mcp_profile_page() ) {
			return;
		}

		mcp_handle_profile_view();
	}

	/**
	 * Check that user can edit their account.
	 */
	public static function account_redirect() {
		global $current_user;
		if ( ! is_mcp_account_page() || ! is_user_logged_in() ) {
			return;
		}
		if ( ! current_user_can( 'mcp_edit_account' ) ) {
			exit( wp_safe_redirect( add_query_arg( 'unauthorized', 'true', home_url() ) ) );
		}
	}

	/**
	 * Log in as another user.
	 */
	public static function login_as_user() {
		// This feature is only users with edit users capability.
		if ( ! current_user_can( 'mcp_edit_users' ) || empty( $_REQUEST[ 'mcp_logon' ] ) || empty( $_REQUEST[ 'mcp_id' ] ) ) {
			return;
		}

		// Compare the md5 hashes.
		if ( $_REQUEST[ 'mcp_logon' ] == mcp_md5( $_REQUEST[ 'mcp_id' ] ) ) {
			wp_logout();

			wp_set_current_user( $_REQUEST[ 'mcp_id' ] );
			wp_set_auth_cookie( $_REQUEST[ 'mcp_id' ], true );

			wp_safe_redirect( memberchimp_get_profile_url( $_REQUEST[ 'mcp_id' ] ) );
			exit;
		}
	}

	/**
	 * Fired when the user attempt to enter a WP admin screen.
	 */
	public static function wp_admin_screen() {
		global $pagenow;
		if ( $pagenow != 'wp-login.php' ) {
			return;
		}

		$action = ! empty( $_REQUEST[ 'action' ] ) ? mcp_clean( $_REQUEST[ 'action' ] ) : '';

		switch( $action ) :

			case 'register' :
				$block_register = get_option( 'memberchimp_block_wpregister' );
				if ( $block_register === 'yes' ) {
					$check = is_mcp_page_has_form( mcp_get_page_id( 'register' ), 'register' );
					if ( $check ) {
						exit( wp_safe_redirect( apply_filters( 'mcp_default_frontend_register_redirect', mcp_register_url() ) ) );
					}
				}
			break;

			case 'lostpassword' :
				$block_lostpassword = get_option( 'memberchimp_block_wplostpassword' );
				if ( $block_lostpassword === 'yes' ) {
					$check = is_mcp_page_has_form( mcp_get_page_id( 'lostpassword' ), 'lostpassword' );
					if ( $check ) {
						exit( wp_safe_redirect( apply_filters( 'mcp_default_frontend_lostpassword_redirect', mcp_lostpassword_url() ) ) );
					}
				}
			break;

			// Fix logout through WordPress.
			case 'logout' :

			break;

			default :
				$block_login = get_option( 'memberchimp_block_wplogin' );
				if ( $block_login === 'yes' ) {
					$check = is_mcp_page_has_form( mcp_get_page_id( 'login' ), 'login' );
					if ( $check ) {
						exit( wp_safe_redirect( apply_filters( 'mcp_default_frontend_login_redirect', mcp_login_url() ) ) );
					}
				}
			break;

		endswitch;
	}

	/**
	 * Maybe validate username change.
	 */
	public static function maybe_validate_username( $postdata ) {
		global $the_form, $the_user;

		if ( empty( $postdata[ 'user_login' ] ) ) {
			return;
		}

		if ( get_option( 'memberchimp_allow_user_login_change' ) != 'yes' ) {
			return;
		}

		if ( sanitize_user( $postdata[ 'user_login' ] ) == $the_user->user_login ) {
			return;
		}

		$value = sanitize_user( $postdata[ 'user_login' ] );

		if ( validate_username( $value ) == false ) {
			mcp_add_notice( esc_html__( 'Your username can have letters, numbers and underscore.', 'memberchimp' ), 'error', 'user_login' );
		}

		if ( username_exists( $value ) ) {
			mcp_add_notice( esc_html__( 'Username has already been taken.', 'memberchimp' ), 'error', 'user_login' );
		}

		$the_form->postdata[ 'user_login' ] = $value;
	}

	/**
	 * Maybe validate email change.
	 */
	public static function maybe_validate_email( $postdata ) {
		global $the_form, $the_user;

		if ( empty( $postdata[ 'user_email' ] ) ) {
			return;
		}

		if ( $email = sanitize_email( $postdata[ 'user_email' ] ) == $the_user->user_email ) {

			unset( $the_form->postdata[ 'user_email' ] );

		} else {

			$the_form->postdata[ 'user_email' ] == $email;

		}
	}

	/**
	 * Maybe validate password change.
	 */
	public static function maybe_validate_password( $postdata ) {
		global $the_form, $the_user;

		if ( empty( $postdata[ 'user_pass' ] ) ) {
			mcp_add_notice( esc_html__( 'Please enter your current password.', 'memberchimp' ), 'error', 'user_pass' );
		} elseif ( empty( $postdata[ 'new_password' ] ) ) {
			mcp_add_notice( esc_html__( 'You must enter a new password to change it.', 'memberchimp' ), 'error', 'global' );
		} elseif ( $postdata[ 'new_password' ] !== $postdata[ 'verify_new_password' ] ) {
			mcp_add_notice( esc_html__( 'Your new password must be confirmed correctly.', 'memberchimp' ), 'error', 'global' );
		}

		$check_password = wp_check_password( $postdata[ 'user_pass' ], $the_user->user_pass, $the_user->ID );

		if ( ! $check_password ) {
			mcp_add_notice( esc_html__( 'Your password is incorrect.', 'memberchimp' ), 'error', 'user_pass' );
		}
	}

}

MCP_Form_Handler::init();