<?php
/**
 * Forms Core.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'MCP_Abstract_Post', false ) ) {
	include_once 'abstracts/abstract-class-mcp-post.php';
}

/**
 * MCP_Form class.
 */
class MCP_Form extends MCP_Abstract_Post {

	/**
	 * Post type.
	 */
	public $post_type = 'mcp_form';

	/**
	 * Meta keys.
	 */
	public $internal_meta_keys = array(
		'type',
		'role',
		'rows',
		'fields',
		'row_count',
		'cols',
		'use_ajax',
		'force_role',
		'icons',
		'endpoint',
		'redirect',
		'redirect_uri',
		'hide_cover',
	);

	/**
	 * Password generated.
	 */
	public $password_generated = null;

	/**
	 * Holds the saved query parameter value.
	 */
	public $saved = 'true';

	/**
	 * Define if the form needs to be cleared.
	 */
	public $cleardata = null;

	/**
	 * Get form endpoint.
	 */
	public function get_endpoint() {
		$endpoint = $this->endpoint;

		if ( null == $endpoint ) {
			$endpoint = $this->type;
		}

		return wp_unslash( $endpoint );
	}

	/**
	 * Does the form have fields?
	 */
	public function has_fields() {
		if ( ! empty( $this->fields ) ) {
			return true;
		}
		return false;
	}

	/**
	 * Get fields per specific row.
	 */
	public function fields_in( $row = 0, $col = 0 ) {
		if ( empty( $this->fields ) ) {
			return;
		}

		// Fix column start at zero.
		$col = $col + 1;

		// Allow output to be filtered.
		$this->fields = apply_filters( 'mcp_get_form_fields', $this->fields, $this );

		return array_filter( ( array ) $this->fields, function( $val ) use ($row, $col) {
			return ( $val['row'] == absint( $row ) && $val['col'] == $col );
		} );
	}

	/**
	 * Custom save action.
	 */
	public function _save( $props ) {
		if ( ! empty( $props[ 'endpoint' ] ) ) {
			if ( $props[ 'endpoint' ] !== $this->endpoint ) {
				delete_option( 'memberchimp_account_' . mcp_sanitize_title( $this->endpoint ) . '_form' );
			}
			update_option( 'memberchimp_account_' . mcp_sanitize_title( $props[ 'endpoint' ] ) . '_form', $this->id );
		}

		// Save the role.
		if ( isset( $props[ 'type' ] ) && $props[ 'type' ] == 'profile' ) {
			if ( isset( $props[ 'role' ] ) && $props[ 'role' ] != $this->role ) {
				delete_option( 'memberchimp_profile_form_' . $this->role );
			}
			if ( isset( $props[ 'force_role' ] ) && $props[ 'force_role' ] == 'yes' && $props[ 'role' ] ) {
				update_option( 'memberchimp_profile_form_' . $props[ 'role' ], $this->id );
			}
		}
	}

	/**
	 * Get the role title.
	 */
	public function get_role_title() {
		$role = $this->role;
		$roles = memberchimp_get_roles();

		return isset( $roles[ $role ] ) ? $roles[ $role ] : '';
	}

	/**
	 * Fires redirect after profile edit.
	 */
	public function profile_redirect( $user = null ) {
		if ( empty( $user ) ) {
			$user = mcp_get_active_profile_id();
		}

		$redirect = add_query_arg( array( 'saved' => $this->saved ), memberchimp_get_profile_url( $user ) );

		return $this->get_redirect( $redirect );
	}

	/**
	 * Fires redirect after account edit.
	 */
	public function account_redirect() {
		$redirect = add_query_arg( array( 'saved' => $this->saved ), mcp_get_account_endpoint_url( $this->get_endpoint() ) );

		return $this->get_redirect( $redirect );
	}

	/**
	 * Get the redirect depending on type of request. AJAX or standard.
	 */
	public function get_redirect( $redirect ) {
		if ( is_admin() && defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			$this->js_redirect = $redirect;
		} else {
			// Should we use wp_safe_redirect? For now, we use this to allow external redirection.
			wp_redirect( $redirect );
			exit;
		}
	}

	/**
	 * Check if form has errors.
	 */
	public function has_errors() {
		return ! empty( $this->error_fields );
	}

	/**
	 * Add new error.
	 */
	public function error( $key ) {
		if ( ! $this->has_error( $key ) ) {
			$this->error_fields[] = $key;
		}
	}

	/**
	 * Check if key is an error.
	 */
	public function has_error( $key ) {
		return in_array( $key, $this->error_fields );
	}

	/**
	 * Get error fields.
	 */
	public function get_error_fields() {
		$error_fields = null;
		if ( ! empty( $this->error_fields ) ) {
			foreach( $this->error_fields as $key ) {
				$error_fields[] = '.' . $key . '_field';
			}
		}
		return $error_fields;
	}

	/**
	 * Get form fields.
	 */
	public function get_fields() {
		return apply_filters( 'mcp_get_form_fields', $this->fields, $this );
	}

	/**
	 * Validation of user input.
	 */
	public function validate() {
		if ( empty( $this->get_fields() ) ) {
			return;
		}

		foreach( $this->get_fields() as $key => $array ) {
			$this->validate_input( $array['data'] );
		}

		// Add custom validation for a specific form type.
		do_action( 'memberchimp_' . $this->type . '_validation', $this->get_postdata() );

		// Add custom validation for a specific endpoint.
		if ( $this->endpoint ) {
			do_action( 'memberchimp_' . mcp_sanitize_title( $this->endpoint ) . '_validation', $this->get_postdata() );
		}
	}

	/**
	 * Default validation.
	 */
	public function validate_input( $data ) {
		extract( $data );

		$update = true;
		$value  = ! isset( $_REQUEST[ $key ] ) ? '' : $_REQUEST[ $key ];

		// Checks based on key.
		switch( $key ) {
			case 'user_login' :
				if ( ! $value ) {
					if ( $this->type == 'register' ) {
						mcp_add_notice( esc_html__( 'Please enter a username.', 'memberchimp' ), 'error', $key );
					} elseif ( $this->type == 'login' ) {
						mcp_add_notice( esc_html__( 'Please enter your username.', 'memberchimp' ), 'error', $key );
					}
				}
			break;
			case 'user_pass' :
				if ( ! $value ) {
					if ( $this->type == 'register' ) {
						mcp_add_notice( esc_html__( 'Please enter a password.', 'memberchimp' ), 'error', $key );
					} elseif ( $this->type == 'login' ) {
						mcp_add_notice( esc_html__( 'Please enter your password.', 'memberchimp' ), 'error', $key );
					}
				}
			break;
			case 'user_email' :
				if ( ! $value ) {
					mcp_add_notice( esc_html__( 'Please enter your email.', 'memberchimp' ), 'error', $key );
				}
			break;
		}

		// Checks based on type.
		switch( $type ) {
			case 'email' :
				if ( ! is_email( trim( $value ) ) && ! empty( $value ) ) {
					mcp_add_notice( esc_html__( 'Please enter a valid email.', 'memberchimp' ), 'error', $key );
				}
				$value = sanitize_email( $value );
				// Check if email is blocked.
				if ( mcp_email_blocked( $value, $data ) ) {
					mcp_add_notice( esc_html__( 'You are not allowed to use this email.', 'memberchimp' ), 'error', $key );
				}
			break;
			case 'phone' :
				if ( ! empty( $value ) && preg_match( '/[a-z]/i', $value ) ) {
					mcp_add_notice( esc_html__( 'Invalid phone number entered.', 'memberchimp' ), 'error', $key );
				}
				$value = mcp_clean( wp_unslash( $value ) );
			break;
			case 'toggle' :
				$value = isset( $_REQUEST[ $key ] );
			break;
			case 'file' :
			case 'image' :
				$value = $this->add_file_metadata( $data );
				if ( empty( $value ) ) {
					$update = false;
				}
			break;
		}

		// Checks based on form type.
		switch( $this->type ) {
			case 'register' :
				if ( $key == 'user_email' ) {
					if ( $user_id = email_exists( $value ) ) {
						// Email exists but have not confirmed. Try to send a new code.
						if ( mcp_user_email_unconfirmed( $value ) ) {
							$user = get_userdata( $user_id );

							$confirmation_code = MCP_Shortcode_Form_Register::create_confirmation_code( $user );

							wp_safe_redirect( add_query_arg( array( 'checkmail' => 'true' ) ) );
							exit;
						}
						mcp_add_notice( esc_html__( 'Email has already been taken.', 'memberchimp' ), 'error', $key );
					}
				}
				if ( $key == 'user_login' ) {
					if ( username_exists( $value ) ) {
						mcp_add_notice( esc_html__( 'Username has already been taken.', 'memberchimp' ), 'error', $key );
					}
					if ( validate_username( $value ) == false ) {
						mcp_add_notice( esc_html__( 'Your username can have letters, numbers and underscore.', 'memberchimp' ), 'error', $key );
					}
				}
			break;
		}

		// Check if required.
		if ( ! $value && ! empty( $is_required ) ) {
			if ( ! empty( $custom_error ) ) {
				mcp_add_notice( $custom_error, 'error', $key );
			} elseif ( ! empty( $label ) ) {
				mcp_add_notice( sprintf( esc_html__( '%s is required.', 'memberchimp' ), $label ), 'error', $key );
			} else {
				mcp_add_notice( esc_html__( 'An error has occurred.', 'memberchimp' ), 'error', $key );
			}
		}

		// Add custom validation for a specific key.
		do_action( 'memberchimp_validate_input_' . $key, $data );

		// Add the key/value to the final update array.
		if ( apply_filters( 'memberchimp_validate_input_flag', $update, $key, $data ) ) {
			$this->postdata[ $key ] = $value;
		}
	}

	/**
	 * Add file metadata.
	 */
	public function add_file_metadata( $data ) {
		extract( $data );

		$array = array();

		if ( isset( $_FILES[ $key ] ) && $_FILES[ $key ]['size'] > 0 && $_FILES[ $key ]['error'] == 0 ) {
			if ( ! empty( $data['mimes'] ) && $data['type'] == 'file' ) {
				$data['mimes'] = explode( ',', $data['mimes'] );
				$mimes = get_allowed_mime_types();
				foreach( $data['mimes'] as $index => $mime ) {
					unset( $data['mimes'][ $index ] );
					if ( isset( $mimes[ $mime ] ) ) {
						$data['mimes'][ $mime ] = $mimes[ $mime ];
					}
				}
			}
			$array = array_merge( $_FILES[ $key ], array( 'field' => $data ) );
		}

		return $array;
	}

	/**
	 * Get post data.
	 */
	public function get_postdata() {
		return ! empty( $this->postdata ) ? apply_filters( 'memberchimp_get_postdata', $this->postdata, $this ) : null;
	}

	/**
	 * Get secondary button url.
	 */
	public function get_second_button_url( $url = null ) {
		switch( $this->type ) :
			case 'login' :
				$url = mcp_register_url();
			break;
			case 'register' :
			case 'lostpassword' :
				$url = mcp_login_url();
			break;
			default :
			break;
		endswitch;

		return apply_filters( 'memberchimp_secondary_button_url', $url, $this );
	}

	/**
	 * Add custom form support.
	 */
	public function add_custom( $type, $hidden_fields = array() ) {
		$this->type = mcp_clean( wp_unslash( $type ) );
		$method = "get_custom_{$this->type}_fields";

		if ( method_exists( $this, $method ) ) {
			$this->$method();
		} else {

			// Allow plugins and 3rd party to add their custom fields.
			do_action( 'memberchimp_' . $this->type . '_custom_fields' );
		}

		// Add hidden inputs.
		if ( ! empty( $hidden_fields ) ) {
			$this->hidden_fields = $hidden_fields;
		}
	}

	/**
	 * Get password reset custom fields.
	 */
	public function get_custom_password_reset_fields() {
		unset( $this->fields );
		$this->fields[] = array(
			'data' 	=> array(
				'key'			=> 'password_1',
				'label'			=> esc_html__( 'New password', 'memberchimp' ),
				'type'			=> 'password',
				'hide_toggle' 	=> true,
			),
			'row'  	=> 1,
			'col'	=> 1,
		);

		$this->fields[] = array(
			'data' 	=> array(
				'key'			=> 'password_2',
				'label'			=> esc_html__( 'Confirm new password', 'memberchimp' ),
				'type'			=> 'password',
				'hide_toggle' 	=> true,
			),
			'row'  	=> 1,
			'col'	=> 1,
		);
	}

}