<?php
/**
 * Handle frontend scripts
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MCP_Frontend_Scripts class.
 */
class MCP_Frontend_Scripts {

	/**
	 * Contains an array of script handles registered.
	 */
	private static $scripts = array();

	/**
	 * Contains an array of script handles registered.
	 */
	private static $styles = array();

	/**
	 * Contains an array of script handles localized.
	 */
	private static $wp_localize_scripts = array();

	/**
	 * Hook in methods.
	 */
	public static function init() {
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'load_scripts' ) );
		add_action( 'wp_print_scripts', array( __CLASS__, 'localize_printed_scripts' ), 5 );
		add_action( 'wp_print_footer_scripts', array( __CLASS__, 'localize_printed_scripts' ), 5 );

		add_action( 'memberchimp_update_options_style', array( __CLASS__, 'save_styles' ) );
	}

	/**
	 * Get styles for the frontend.
	 */
	public static function get_styles() {
		return apply_filters(
			'memberchimp_enqueue_styles',
			array(
				'memberchimp-layout'	=> array(
					'src'     => self::get_asset_url( 'assets/css/memberchimp-layout.css' ),
					'deps'    => '',
					'version' => MCP_VERSION,
					'media'   => 'all',
					'has_rtl' => true,
				),
				'memberchimp-style'	=> array(
					'src'     => self::get_theme(),
					'deps'    => '',
					'version' => MCP_VERSION,
					'media'   => 'all',
					'has_rtl' => true,
				),
				'memberchimp-general'	=> array(
					'src'     => self::get_asset_url( 'assets/css/memberchimp.css' ),
					'deps'    => '',
					'version' => MCP_VERSION,
					'media'   => 'all',
					'has_rtl' => true,
				),
				'memberchimp-media'	=> array(
					'src'     => self::get_asset_url( 'assets/css/memberchimp-media.css' ),
					'deps'    => '',
					'version' => MCP_VERSION,
					'media'   => 'all',
					'has_rtl' => true,
				),
			)
		);
	}

	/**
	 * Get correct theme to load.
	 */
	public static function get_theme() {
		$custom = get_option( 'memberchimp_style' );
		if ( $custom && file_exists( mcp_plugin_uploads_path() . '/' . wp_basename( $custom ) ) ) {
			return $custom;
		} else {
			return self::get_asset_url( 'assets/css/memberchimp-style.css' );
		}
	}

	/**
	 * Return asset URL.
	 */
	private static function get_asset_url( $path ) {
		return apply_filters( 'memberchimp_get_asset_url', plugins_url( $path, MCP_PLUGIN_FILE ), $path );
	}

	/**
	 * Register a script for use.
	 */
	private static function register_script( $handle, $path, $deps = array( 'jquery' ), $version = MCP_VERSION, $in_footer = true ) {
		self::$scripts[] = $handle;
		wp_register_script( $handle, $path, $deps, $version, $in_footer );
	}

	/**
	 * Register and enqueue a script for use.
	 */
	private static function enqueue_script( $handle, $path = '', $deps = array( 'jquery' ), $version = MCP_VERSION, $in_footer = true ) {
		if ( ! in_array( $handle, self::$scripts, true ) && $path ) {
			self::register_script( $handle, $path, $deps, $version, $in_footer );
		}
		wp_enqueue_script( $handle );
	}

	/**
	 * Register a style for use.
	 */
	private static function register_style( $handle, $path, $deps = array(), $version = MCP_VERSION, $media = 'all', $has_rtl = false ) {
		self::$styles[] = $handle;
		wp_register_style( $handle, $path, $deps, $version, $media );

		if ( $has_rtl ) {
			wp_style_add_data( $handle, 'rtl', 'replace' );
		}
	}

	/**
	 * Register and enqueue a styles for use.
	 */
	private static function enqueue_style( $handle, $path = '', $deps = array(), $version = MCP_VERSION, $media = 'all', $has_rtl = false, $inline = false ) {
		if ( ! in_array( $handle, self::$styles, true ) && $path ) {
			self::register_style( $handle, $path, $deps, $version, $media, $has_rtl, $inline );
		}
		wp_enqueue_style( $handle );

		if ( $inline ) {
			wp_add_inline_style( $handle, self::get_inline_css( $handle ) );
		}
	}

	/**
	 * Register all scripts.
	 */
	private static function register_scripts() {
		$register_scripts = array(
			'jquery-tiptip'		=> array(
				'src'     => self::get_asset_url( 'assets/js/jquery-tiptip/jquery-tiptip.js' ),
				'deps'    => array( 'jquery' ),
				'version' => MCP_VERSION,
			),
			'jquery-toggles'	=> array(
				'src'     => self::get_asset_url( 'assets/js/jquery-toggles/jquery-toggles.js' ),
				'deps'    => array( 'jquery' ),
				'version' => MCP_VERSION,
			),
			'jquery-modal'		=> array(
				'src'     => self::get_asset_url( 'assets/js/jquery-modal/jquery-modal.js' ),
				'deps'    => array( 'jquery' ),
				'version' => MCP_VERSION,
			),
			'jquery-selectize' 	=> array(
				'src'	  => self::get_asset_url( 'assets/js/jquery-selectize/jquery-selectize.js' ),
				'deps'    => array( 'jquery' ),
				'version' => MCP_VERSION,
			),
			'jquery-raty'		=> array(
				'src'	  => self::get_asset_url( 'assets/js/jquery-raty/jquery-raty.js' ),
				'deps'    => array( 'jquery' ),
				'version' => MCP_VERSION,
			),
			'jquery-croppie'	=> array(
				'src'     => self::get_asset_url( 'assets/js/jquery-croppie/jquery-croppie.js' ),
				'deps'    => array( 'jquery' ),
				'version' => MCP_VERSION,
			),
			'jquery-mcp'		=> array(
				'src'     => self::get_asset_url( 'assets/js/jquery-mcp/jquery-mcp.js' ),
				'deps'    => array( 'jquery', 'jquery-tiptip', 'jquery-toggles', 'jquery-modal', 'jquery-selectize', 'jquery-raty' ),
				'version' => MCP_VERSION,
			),
			'mcp-profile'		=> array(
				'src'     => self::get_asset_url( 'assets/js/frontend/profile.js' ),
				'deps'    => array( 'jquery', 'jquery-croppie' ),
				'version' => MCP_VERSION,
			),
			'memberchimp'		=> array(
				'src'     => self::get_asset_url( 'assets/js/frontend/memberchimp.js' ),
				'deps'    => apply_filters( 'memberchimp_javascript_deps', array( 'jquery', 'jquery-mcp' ) ),
				'version' => MCP_VERSION,
			),
		);
		foreach ( $register_scripts as $name => $props ) {
			self::register_script( $name, $props['src'], $props['deps'], $props['version'] );
		}
	}

	/**
	 * Register all styles.
	 */
	private static function register_styles() {
		$register_styles = array(

		);
		foreach ( $register_styles as $name => $props ) {
			self::register_style( $name, $props['src'], $props['deps'], $props['version'], 'all', $props['has_rtl'] );
		}
	}

	/**
	 * Register/queue frontend scripts.
	 */
	public static function load_scripts() {
		global $post;

		if ( ! did_action( 'before_memberchimp_init' ) ) {
			return;
		}

		self::register_scripts();
		self::register_styles();

		// Global frontend scripts.
		self::enqueue_script( 'memberchimp' );

		// Profile.
		if ( is_mcp_profile_page() ) {
			self::enqueue_script( 'mcp-profile' );
		}

		// CSS Styles.
		$enqueue_styles = self::get_styles();
		if ( $enqueue_styles ) {
			foreach ( $enqueue_styles as $handle => $args ) {

				if ( ! isset( $args['has_rtl'] ) ) {
					$args['has_rtl'] = false;
				}
				if ( ! isset( $args['inline'] ) ) {
					$args['inline'] = false;
				}

				self::enqueue_style( $handle, $args['src'], $args['deps'], $args['version'], $args['media'], $args['has_rtl'], $args['inline'] );
			}
		}
	}

	/**
	 * Localize a script once.
	 */
	private static function localize_script( $handle ) {
		if ( ! in_array( $handle, self::$wp_localize_scripts, true ) && wp_script_is( $handle ) ) {
			$data = self::get_script_data( $handle );

			if ( ! $data ) {
				return;
			}

			$name                        = str_replace( '-', '_', $handle ) . '_params';
			self::$wp_localize_scripts[] = $handle;
			wp_localize_script( $handle, $name, apply_filters( $name, $data ) );
		}
	}

	/**
	 * Return data for script handles.
	 */
	private static function get_script_data( $handle ) {
		global $wp;

		switch ( $handle ) {
			case 'memberchimp':
				$params = array(
					'ajaxurl'   => mcp()->ajax_url(),
					'yes'		=> esc_html__( 'yes', 'memberchimp' ),
					'no'		=> esc_html__( 'no', 'memberchimp' ),
					'svg'		=> mcp()->plugin_url() . '/assets/images/feather-sprite.svg#',
					'nonces'	=> apply_filters( 'mcp_ajax_nonces', array(
						'cancel_pending_email' 	=> wp_create_nonce( 'memberchimp-cancel-email' ),
						'resend_confirmation'	=> wp_create_nonce( 'memberchimp-resend-confirmation' ),
						'save_avatar'			=> wp_create_nonce( 'memberchimp-save-avatar' ),
						'save_cover'			=> wp_create_nonce( 'memberchimp-save-cover' ),
						'remove_avatar'			=> wp_create_nonce( 'memberchimp-remove-avatar' ),
						'remove_cover'			=> wp_create_nonce( 'memberchimp-remove-cover' ),
						'unblock_user'			=> wp_create_nonce( 'memberchimp-unblock-user' ),
						'block_user'			=> wp_create_nonce( 'memberchimp-block-user' ),
						'delete_user'			=> wp_create_nonce( 'memberchimp-delete-user' ),
						'confirmation_code'		=> wp_create_nonce( 'memberchimp-confirmation-code' ),
					) ),
					'saved'		=> apply_filters( 'mcp_saved_messages', array(
						'true'				=> esc_html__( 'Changes have been saved successfully.', 'memberchimp' ),
						'username_changed'  => esc_html__( 'Your username has been changed.', 'memberchimp' ),
						'password_changed'  => esc_html__( 'Your password has been changed.', 'memberchimp' ),
						'confirm_email'		=> esc_html__( 'A confirmation email has been sent to you.', 'memberchimp' ),
						'email_changed'		=> esc_html__( 'Your email has been changed.', 'memberchimp' ),
						'pending_account'	=> esc_html__( 'Your account needs manual review.', 'memberchimp' ),
					) ),
				);
				break;
			default:
				$params = false;
		}

		return apply_filters( 'memberchimp_get_script_data', $params, $handle );
	}

	/**
	 * Localize scripts only when enqueued.
	 */
	public static function localize_printed_scripts() {
		foreach ( self::$scripts as $handle ) {
			self::localize_script( $handle );
		}
	}

	/**
	 * Get inline css.
	 */
	public static function get_inline_css( $handle = '', $mcp_css = '' ) {
		return apply_filters( 'mcp_inline_styles', $mcp_css, $handle );
	}

	/**
	 * Save styles for the frontend.
	 */
	public static function save_styles() {
		if ( isset( $_POST ) ) {
			$data = $_POST;
		}

		$current = array(
			'memberchimp_primary_bg_color'	=> get_option( 'memberchimp_primary_bg_color', '#5b9dd9' ),
			'memberchimp_primary_color'	=> get_option( 'memberchimp_primary_color', '#66757f' ),
			'memberchimp_alert_color'		=> get_option( 'memberchimp_alert_color', '#bb0000' ),
		);

		$data = array_intersect_key( $data, $current );

		// Avoid empty colors.
		foreach( $data as $key => $color ) {
			if ( empty( sanitize_hex_color( $color ) ) ) {
				$data[ $key ] = $current[ $key ];
			}
		}

		$old = self::get_rgb( $current );
		$new = self::get_rgb( array_merge( $current, $data ) );

		self::save_css( $old, $new );

		foreach( $data as $key => $value ) {
			update_option( $key, mcp_clean( $value ) );
		}
	}

	/**
	 * Convert hex color array to rgb color array.
	 */
	public static function get_rgb( $array ) {
		foreach( $array as $key => $hex ) {
			list($r, $g, $b) = sscanf( $hex, "#%02x%02x%02x" );
			$array[ $key ] = "{$r}, {$g}, {$b}";
		}
		return $array;
	}

	/**
	 * Save the css file.
	 */
	public static function save_css( $old = array(), $new = array() ) {
		$contents = @file_get_contents( mcp()->plugin_path() . '/assets/css/memberchimp-style.css' );
		if ( empty( $contents ) ) {
			return;
		}

		add_filter( 'upload_dir', array( __CLASS__, 'upload_dir' ) );

		foreach( $new as $name => $rgb ) {
			$contents = str_replace( $old[ $name ], $new[ $name ], $contents );
		}

		$current = get_option( 'memberchimp_style' );
		if ( $current ) {
			wp_delete_file( mcp_plugin_uploads_path() . '/' . wp_basename( $current ) );
			wp_delete_file( mcp_plugin_uploads_path() . '/' . wp_basename( str_replace( '.css', '-rtl.css', $current ) ) );
		}

		$upload 	= wp_upload_bits( 'memberchimp-style.css', null, apply_filters( 'mcp_new_style_content', $contents ), null );
		$upload_rtl = wp_upload_bits( 'memberchimp-style-rtl.css', null, apply_filters( 'mcp_new_rtl_style_content', $contents ), null );

		if ( isset( $upload['url'] ) ) {
			update_option( 'memberchimp_style', $upload['url'] );
		}

		remove_filter( 'upload_dir', array( __CLASS__, 'upload_dir' ) );
	}

	/**
	 * Filter uploads directory during css file creation.
	 */
	public static function upload_dir( $upload_dir ) {
		$css_upload_dir = array(
			'path'   => $upload_dir['basedir'] . '/mcp_uploads',
			'url'    => $upload_dir['baseurl'] . '/mcp_uploads',
			'subdir' => '/mcp_uploads',
		) + $upload_dir;

		return $css_upload_dir;
	}

}

MCP_Frontend_Scripts::init();