<?php
/**
 * Core Functions.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// Include core functions (available in both admin and frontend).
require MCP_ABSPATH . 'includes/mcp-conditional-functions.php';
require MCP_ABSPATH . 'includes/mcp-formatting-functions.php';
require MCP_ABSPATH . 'includes/mcp-date-functions.php';
require MCP_ABSPATH . 'includes/mcp-upload-functions.php';
require MCP_ABSPATH . 'includes/mcp-form-functions.php';
require MCP_ABSPATH . 'includes/mcp-field-functions.php';
require MCP_ABSPATH . 'includes/mcp-role-functions.php';
require MCP_ABSPATH . 'includes/mcp-memberlist-functions.php';
require MCP_ABSPATH . 'includes/mcp-page-functions.php';
require MCP_ABSPATH . 'includes/mcp-account-functions.php';
require MCP_ABSPATH . 'includes/mcp-profile-functions.php';
require MCP_ABSPATH . 'includes/mcp-user-functions.php';
require MCP_ABSPATH . 'includes/mcp-avatar-functions.php';
require MCP_ABSPATH . 'includes/mcp-cover-functions.php';
require MCP_ABSPATH . 'includes/mcp-dropdown-functions.php';
require MCP_ABSPATH . 'includes/mcp-locale-functions.php';
require MCP_ABSPATH . 'includes/mcp-cron-functions.php';
require MCP_ABSPATH . 'includes/mcp-widget-functions.php';

/**
 * Returns true when we're displaying a loop.
 */
function mcp_is_in_loop() {
	global $the_memberlist;

	$in_loop = false;

	if ( isset( $the_memberlist->_in_loop ) && $the_memberlist->_in_loop == true ) {
		$in_loop = true;
	}

	return apply_filters( 'mcp_is_in_loop', $in_loop );
}

/**
 * Return a list of plugin specific post types.
 */
function mcp_get_post_types() {
	return apply_filters( 'mcp_get_post_types', array( 'mcp_form', 'mcp_field', 'mcp_role', 'mcp_memberlist' ) );
}

/**
 * Define a constant if it is not already defined.
 */
function mcp_maybe_define_constant( $name, $value ) {
	if ( ! defined( $name ) ) {
		define( $name, $value );
	}
}

/**
 * Return the html selected attribute if stringified $value is found in array of stringified $options
 * or if stringified $value is the same as scalar stringified $options.
 */
function mcp_selected( $value, $options ) {
	if ( is_array( $options ) ) {
		$options = array_map( 'strval', $options );
		return selected( in_array( (string) $value, $options, true ), true, false );
	}

	return selected( $value, $options, false );
}

/**
 * Display a SVG icon from the sprite.
 */
function mcp_svg_icon( $icon = '' ) {
	$html = '<svg class="feather"><use xlink:href="'. mcp()->plugin_url() . '/assets/images/feather-sprite.svg#' . esc_html( $icon ) . '" /></svg>';

	// can be used for custom icon output.
	return apply_filters( 'mcp_svg_icon_html', $html, $icon );
}

/**
 * Display a help tip.
 */
function mcp_help_tip( $tip, $allow_html = false ) {
	if ( $allow_html ) {
		$tip = mcp_sanitize_tooltip( $tip );
	} else {
		$tip = esc_attr( $tip );
	}

	return '<span class="memberchimp-help-tip" data-tip="' . $tip . '">' . mcp_svg_icon( 'help-circle' ) . '</span>';
}

/**
 * Get template part.
 */
function mcp_get_template_part( $slug, $name = '' ) {
	global $the_form, $the_user, $the_memberlist, $logged_user;

	$template = '';

	// Look in yourtheme/slug-name.php and yourtheme/memberchimp/slug-name.php.
	if ( $name && ! MCP_TEMPLATE_DEBUG_MODE ) {
		$template = locate_template( array( "{$slug}-{$name}.php", mcp()->template_path() . "{$slug}-{$name}.php" ) );
	}

	// Get default slug-name.php.
	if ( ! $template && $name && file_exists( mcp()->plugin_path() . "/templates/{$slug}-{$name}.php" ) ) {
		$template = mcp()->plugin_path() . "/templates/{$slug}-{$name}.php";
	}

	// If template file doesn't exist, look in yourtheme/slug.php and yourtheme/memberchimp/slug.php.
	if ( ! $template && ! MCP_TEMPLATE_DEBUG_MODE ) {
		$template = locate_template( array( "{$slug}.php", mcp()->template_path() . "{$slug}.php" ) );
	}

	// Allow 3rd party plugins to filter template file from their plugin.
	$template = apply_filters( 'mcp_get_template_part', $template, $slug, $name );

	if ( $template ) {
		load_template( $template, false );
	}
}

/**
 * Get other templates (e.g. product attributes) passing attributes and including the file.
 */
function mcp_get_template( $template_name, $args = array(), $template_path = '', $default_path = '' ) {
	global $the_form, $the_user, $the_memberlist, $logged_user;

	if ( ! empty( $args ) && is_array( $args ) ) {
		extract( $args );
	}

	$located = mcp_locate_template( $template_name, $template_path, $default_path );

	if ( ! file_exists( $located ) ) {
		return;
	}

	// Allow 3rd party plugin filter template file from their plugin.
	$located = apply_filters( 'mcp_get_template', $located, $template_name, $args, $template_path, $default_path );

	do_action( 'memberchimp_before_template_part', $template_name, $template_path, $located, $args );

	include $located;

	do_action( 'memberchimp_after_template_part', $template_name, $template_path, $located, $args );
}

/**
 * Like mcp_get_template, but returns the HTML instead of outputting.
 */
function mcp_get_template_html( $template_name, $args = array(), $template_path = '', $default_path = '' ) {
	global $the_form, $the_user, $the_memberlist, $logged_user;

	ob_start();
	mcp_get_template( $template_name, $args, $template_path, $default_path );
	return ob_get_clean();
}

/**
 * Locate a template and return the path for inclusion.
 */
function mcp_locate_template( $template_name, $template_path = '', $default_path = '' ) {
	global $the_form, $the_user, $the_memberlist, $logged_user;

	if ( ! $template_path ) {
		$template_path = mcp()->template_path();
	}

	if ( ! $default_path ) {
		$default_path = mcp()->plugin_path() . '/templates/';
	}

	// Look within passed path within the theme - this is priority.
	$template = locate_template(
		array(
			trailingslashit( $template_path ) . $template_name,
			$template_name,
		)
	);

	// Get default template/.
	if ( ! $template || MCP_TEMPLATE_DEBUG_MODE ) {
		$template = $default_path . $template_name;
	}

	// Return what we found.
	return apply_filters( 'memberchimp_locate_template', $template, $template_name, $template_path );
}

/**
 * Get scope for viewed form. Should be edit or view.
 */
function mcp_get_scope() {
	global $the_form;

	$scope = 'edit';

	if ( $the_form->type == 'profile' ) {
		// only enable editing profile forms when edit mode is turned on.
		if ( get_query_var( 'mcp_tab' ) != 'edit' ) {
			$scope = 'view';
		}
	}

	return apply_filters( 'mcp_get_scope', $scope );
}

/**
 * Sort values based on ascii, usefull for special chars in strings.
 */
function mcp_ascii_uasort_comparison( $a, $b ) {
	if ( function_exists( 'iconv' ) && defined( 'ICONV_IMPL' ) && @strcasecmp( ICONV_IMPL, 'unknown' ) !== 0 ) {
		$a = @iconv( 'UTF-8', 'ASCII//TRANSLIT//IGNORE', $a );
		$b = @iconv( 'UTF-8', 'ASCII//TRANSLIT//IGNORE', $b );
	}
	return strcmp( $a, $b );
}

/**
 * Array insert before a key.
 */
function mcp_array_insert_before( $key, array &$array, $new_key, $new_value ) {
	if ( array_key_exists( $key, $array ) ) {
		$new = array();
		foreach ( $array as $k => $value ) {
			if ( $k === $key ) {
				$new[$new_key] = $new_value;
			}
			$new[$k] = $value;
		}
		return $new;
	}
	return false;
}

/**
 * Array insert after a key.
 */
function mcp_array_insert_after( $key, array &$array, $new_key, $new_value ) {
	if ( array_key_exists( $key, $array ) ) {
		$new = array();
		foreach ( $array as $k => $value ) {
			$new[$k] = $value;
			if ( $k === $key ) {
				$new[$new_key] = $new_value;
			}
		}
		return $new;
	}
	return false;
}

/**
 * Search array and get first matched.
 */
function mcp_array_search( $search, $array ){
    foreach( $array as $item ) {
		foreach( $item as $key => $val ) {
			if ( is_array( $val ) ) {
				foreach( $val as $metakey => $metavalue ) {
					if ( $metavalue == $search ) {
						return $item;
					}
				}
			}
		}
    }
	return false;
}

/**
 * Get maximum value for a column in an array.
 */
function mcp_get_max_column( $array, $column ) {
	$counts = array();
	foreach( $array as $key => $value ) {
		if ( isset( $value[ $column ] ) ) {
			$counts[] = $value[ $column ];
		}
	}
	return $max = max( $counts );
}

/**
 * Get rating hints.
 */
function mcp_get_rating_hints( $field = null ) {

	$defaults = apply_filters( 'mcp_get_default_rating_hints', array(
		__( 'Poor', 'memberchimp' ),
		__( 'Below Average', 'memberchimp' ),
		__( 'Average', 'memberchimp' ),
		__( 'Good', 'memberchimp' ),
		__( 'Excellent', 'memberchimp' ),
	) );

	if ( ! empty( $field[ 'ratings' ] ) ) {
		$ratings = mcp_comma_separated_string( $field[ 'ratings' ] );
	} else {
		$ratings = implode( ',', $defaults );
	}

	return apply_filters( 'mcp_get_rating_hints', $ratings );
}

/**
 * Get a rating hint based on score.
 */
function mcp_get_rating_hint( $score = 0, $field = null ) {
	if ( ! $score ) {
		return;
	}
	$hints = mcp_get_rating_hints( $field );
	$hints = explode( ',', $hints );

	return apply_filters( 'mcp_get_rating_hint', $hints[ $score - 1 ], $score, $field );
}

/**
 * Outputs a "back" link so admin screens can easily jump back a page.
 */
function mcp_back_link( $label, $url ) {
	echo '<small class="mcp-admin-breadcrumb"><a href="' . esc_url( $url ) . '" aria-label="' . esc_attr( $label ) . '">' . esc_html__( '&#8592; Back', 'memberchimp' ) . '</a></small>';
}

/**
 * Queue some JavaScript code to be output in the footer.
 */
function mcp_enqueue_js( $code ) {
	global $mcp_queued_js;

	if ( empty( $mcp_queued_js ) ) {
		$mcp_queued_js = '';
	}

	$mcp_queued_js .= "\n" . $code . "\n";
}

/**
 * Output any queued javascript code in the footer.
 */
function mcp_print_js() {
	global $mcp_queued_js;

	if ( ! empty( $mcp_queued_js ) ) {
		// Sanitize.
		$mcp_queued_js = wp_check_invalid_utf8( $mcp_queued_js );
		$mcp_queued_js = preg_replace( '/&#(x)?0*(?(1)27|39);?/i', "'", $mcp_queued_js );
		$mcp_queued_js = str_replace( "\r", '', $mcp_queued_js );

		$js = "<!-- Member Chimp JavaScript -->\n<script type=\"text/javascript\">\njQuery(function($) { $mcp_queued_js });\n</script>\n";

		echo apply_filters( 'memberchimp_queued_js', $js ); // WPCS: XSS ok.

		unset( $mcp_queued_js );
	}
}

/**
 * Switch to site language.
 */
function mcp_switch_to_site_locale() {
	if ( function_exists( 'switch_to_locale' ) ) {
		switch_to_locale( get_locale() );

		// Filter on plugin_locale so load_plugin_textdomain loads the correct locale.
		add_filter( 'plugin_locale', 'get_locale' );

		// Init locale.
		mcp()->load_plugin_textdomain();
	}
}

/**
 * Switch language to original.
 */
function mcp_restore_locale() {
	if ( function_exists( 'restore_previous_locale' ) ) {
		restore_previous_locale();

		// Remove filter.
		remove_filter( 'plugin_locale', 'get_locale' );

		// Init locale.
		mcp()->load_plugin_textdomain();
	}
}

/**
 * Checks if a provided token is valid.
 */
function mcp_is_invalid_token( $key, $stored ) {
	return empty( $stored[ 'token' ] ) || empty( $stored[ 'expiry' ] ) || $stored[ 'token' ] !== $key || ( $stored[ 'expiry' ] - time() <= 0 );
}

/**
 * Same as wp_get_schedules() but orders the schedules by interval.
 */
function mcp_get_schedules() {
	$schedules = wp_get_schedules();

	uasort( $schedules, function( $a, $b ) {
		return $a[ 'interval' ] - $b[ 'interval' ];
	} );

	$schedules = array_merge( array_combine( array_keys( $schedules ), array_column( $schedules, 'display' ) ), array( '' => esc_html__( 'Never', 'memberchimp' ) ) );

	return apply_filters( 'mcp_get_schedules', $schedules );
}

/**
 * Generate a rand hash.
 */
function mcp_rand_hash() {
	if ( ! function_exists( 'openssl_random_pseudo_bytes' ) ) {
		return sha1( wp_rand() );
	}

	return bin2hex( openssl_random_pseudo_bytes( 20 ) );
}

/**
 * MC API - Hash.
 */
function mcp_api_hash( $data ) {
	return hash_hmac( 'sha256', $data, 'mcp-api' );
}

/**
 * Checks if email or domain is blocked.
 */
function mcp_email_blocked( $email, $data ) {

	if ( ! empty( $data['blocked_emails'] ) && is_email( $email ) ) {
		$email_parts = explode( '@', $email );
		$user		 = trim( $email_parts[0] );
		$domain 	 = trim( $email_parts[1] );

		$values = explode( '+', $data['blocked_emails'] );
		foreach( $values as $value ) {
			if ( substr( $value, 0, 1 ) === '*' ) {
				if ( strstr( $value, $domain ) ) {
					return true;
					break;
				}
			} elseif ( ! strstr( $value, '@' ) ) {
				if ( strstr( $value, str_replace( '@', '', $domain ) ) ) {
					return true;
					break;
				}
			} elseif ( $value == $email ) {
				return true;
				break;
			} elseif ( substr( $value, 0, strlen( $user ) ) == $user && strstr( $value, '@*' ) ) {
				return true;
				break;
			}
		}
	}

	return false;
}