<?php
/**
 * Formatting Functions.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Clean variables using sanitize_text_field. Arrays are cleaned recursively.
 * Non-scalar values are ignored.
 */
function mcp_clean( $var ) {
	if ( is_array( $var ) ) {
		return array_map( 'mcp_clean', $var );
	} else {
		return is_scalar( $var ) ? sanitize_text_field( $var ) : $var;
	}
}

/**
 * Cleans a string and return a comma separated string.
 */
function mcp_comma_separated_string( $string ) {
	return mcp_clean( trim( str_replace( ', ', ',', str_replace( ' ,', ',', $string ) ) ) );
}

/**
 * Clean lowercase variables and use underscore instead of dash.
 */
function mcp_sanitize_title( $var ) {
	return str_replace( '-', '_', sanitize_title( wp_unslash( $var ) ) );
}

/**
 * Sanitize a string destined to be a tooltip.
 */
function mcp_sanitize_tooltip( $var ) {
	return htmlspecialchars(
		wp_kses(
			html_entity_decode( $var ), array(
				'br'     => array(),
				'em'     => array(),
				'strong' => array(),
				'small'  => array(),
				'span'   => array(),
				'ul'     => array(),
				'li'     => array(),
				'ol'     => array(),
				'p'      => array(),
			)
		)
	);
}

/**
 * Make a string lowercase.
 * Try to use mb_strtolower() when available.
 */
function mcp_strtolower( $string ) {
	return function_exists( 'mb_strtolower' ) ? mb_strtolower( $string ) : strtolower( $string );
}

/**
 * Escape URL for frontend view.
 */
function mcp_esc_url( $url ) {
	$disallowed = array( 'http://', 'https://' );
	foreach( $disallowed as $d ) {
		if ( strpos( $url, $d ) === 0 ) {
			return str_replace( $d, '', $url );
		}
	}
	return esc_url( $url );
}

/**
 * Implode and escape HTML attributes for output.
 */
function mcp_implode_html_attributes( $raw_attributes ) {
	$attributes = array();
	foreach ( $raw_attributes as $name => $value ) {
		$attributes[] = esc_attr( $name ) . '="' . esc_attr( $value ) . '"';
	}
	return implode( ' ', $attributes );
}

/**
 * Allowed tags for textarea content.
 */
function mcp_textarea_allowed_tags() {
	$allowed_tags = array();

	return apply_filters( 'mcp_textarea_allowed_tags', $allowed_tags );
}

/**
 * Converts array to encoded string.
 */
function mcp_array_to_encoded_str( $array ) {
	return rtrim( strtr( base64_encode( @gzdeflate( serialize( $array ), 9 ) ), '+/', '-_' ), '=' );
}

/**
 * Converts encoded string back to an array.
 */
function mcp_encoded_str_to_array( $query ) {
	return unserialize( @gzinflate( base64_decode( strtr( mcp_clean( $query ), '-_', '+/' ) ) ) );
}

/**
 * Convert seconds to hours.
 */
function mcp_seconds_to_hours( $seconds ) {
	$hours = floor( $seconds / 3600 );

	return $hours > 1 ? $hours . ' hours ago' : '1 hour ago';
}

/**
 * Returns a more compact md5 hashing.
 */
function mcp_md5( $string ) {
	return substr( base_convert( md5( $string ), 16, 32 ), 0, 12 );
}

/**
 * Generates a unique ID to use for file uploads.
 */
function mcp_unique_filename() {
	return apply_filters( 'mcp_unique_filename', mcp_md5( uniqid() . microtime() . mt_rand() ) );
}

/**
 * Detect if we should use a light or dark color on a background color.
 */
function mcp_light_or_dark( $color, $dark = '#202020', $light = '#FFFFFF' ) {
	return mcp_hex_is_light( $color ) ? $dark : $light;
}

/**
 * Determine whether a hex color is light.
 */
function mcp_hex_is_light( $color ) {
	$hex = str_replace( '#', '', $color );

	$c_r = hexdec( substr( $hex, 0, 2 ) );
	$c_g = hexdec( substr( $hex, 2, 2 ) );
	$c_b = hexdec( substr( $hex, 4, 2 ) );

	$brightness = ( ( $c_r * 299 ) + ( $c_g * 587 ) + ( $c_b * 114 ) ) / 1000;

	return $brightness > 155;
}

/**
 * Make HEX color lighter.
 */
function mcp_hex_lighter( $color, $factor = 30 ) {
	$base  = mcp_rgb_from_hex( $color );
	$color = '#';

	foreach ( $base as $k => $v ) {
		$amount      = 255 - $v;
		$amount      = $amount / 100;
		$amount      = round( $amount * $factor );
		$new_decimal = $v + $amount;

		$new_hex_component = dechex( $new_decimal );
		if ( strlen( $new_hex_component ) < 2 ) {
			$new_hex_component = '0' . $new_hex_component;
		}
		$color .= $new_hex_component;
	}

	return $color;
}

/**
 * Make HEX color darker.
 */
function mcp_hex_darker( $color, $factor = 30 ) {
	$base  = mcp_rgb_from_hex( $color );
	$color = '#';

	foreach ( $base as $k => $v ) {
		$amount      = $v / 100;
		$amount      = round( $amount * $factor );
		$new_decimal = $v - $amount;

		$new_hex_component = dechex( $new_decimal );
		if ( strlen( $new_hex_component ) < 2 ) {
			$new_hex_component = '0' . $new_hex_component;
		}
		$color .= $new_hex_component;
	}

	return $color;
}

/**
 * Convert RGB to HEX.
 */
function mcp_rgb_from_hex( $color ) {
	$color = str_replace( '#', '', $color );
	// Convert shorthand colors to full format, e.g. "FFF" -> "FFFFFF".
	$color = preg_replace( '~^(.)(.)(.)$~', '$1$1$2$2$3$3', $color );

	$rgb      = array();
	$rgb['R'] = hexdec( $color{0} . $color{1} );
	$rgb['G'] = hexdec( $color{2} . $color{3} );
	$rgb['B'] = hexdec( $color{4} . $color{5} );

	return $rgb;
}

/**
 * Generate a token.
 */
function mcp_get_token( $length = 32 ) {
	$token = '';
	$codeAlphabet  = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$codeAlphabet .= 'abcdefghijklmnopqrstuvwxyz';
	$codeAlphabet .= '0123456789';
	for ( $i = 0; $i < $length; $i++ ) {
		$token .= $codeAlphabet[ mcp_crypto_rand_secure( 0, strlen( $codeAlphabet ) ) ];
	}
	return $token;
}

/**
 * Generate a random string for a token.
 */
function mcp_crypto_rand_secure( $min, $max ) {
	$range = $max - $min;
	if ( $range < 0 ) {
		return $min;
	}
	$log    = log( $range, 2);
	$bytes  = ( int )( $log / 8 ) + 1;
	$bits   = ( int ) $log + 1;
	$filter = ( int ) ( 1 << $bits ) - 1;
	do {
		$rnd = hexdec( bin2hex( openssl_random_pseudo_bytes( $bytes ) ) );
		$rnd = $rnd & $filter;
	} while ( $rnd >= $range );
	return $min + $rnd;
}