<?php
/**
 * Profile Functions
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Get profile endpoints.
 */
function mcp_get_profile_endpoints() {
	$endpoints = array(
		'mcp_user'	=> '',
	);

	$endpoints = apply_filters( 'mcp_get_profile_endpoints', ( array ) $endpoints );

	return $endpoints;
}

/**
 * Get active profile ID.
 */
function mcp_get_active_profile_id() {
	$user_id = username_exists( esc_attr( get_query_var( 'mcp_user' ) ) );

	// Get the profile ID when form is being sent via AJAX.
	if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
		$user_id = absint( $_REQUEST[ '_user_id' ] );
	}

	return apply_filters( 'mcp_get_active_profile_id', $user_id );
}

/**
 * Handle the profile view and possible redirect.
 */
function mcp_handle_profile_view() {
	global $current_user;

	$user      = esc_attr( get_query_var( 'mcp_user' ) );
	$user_id   = username_exists( $user );
	$logged_in = is_user_logged_in() ? 1 : 0;

	switch( $logged_in ) {
		case 1 :
			if ( empty( $user ) ) {
				wp_safe_redirect( memberchimp_get_profile_url( $current_user->user_login ) );
				exit;
			}
			if ( ! $user_id ) {
				wp_safe_redirect( add_query_arg( array( 'user_not_found' => 'true' ), home_url() ) );
				exit;
			}
			if ( is_mcp_inactive_user( $user_id ) ) {
				if ( ! current_user_can( 'manage_options' ) && ! current_user_can( 'manage_memberchimp' ) && ! current_user_can( 'mcp_review_users' ) ) {
					wp_safe_redirect( add_query_arg( array( 'inactive' => 'true' ), home_url() ) );
					exit;
				}
			}
			if ( ! current_user_can( 'mcp_view_profiles' ) && $user_id != get_current_user_id() ) {
				wp_safe_redirect( add_query_arg( 'unauthorized', 'true', home_url() ) );
				exit;
			}
		break;
		case 0 :
			if ( empty( $user ) || ! $user_id ) {
				wp_safe_redirect( add_query_arg( array( 'user_not_found' => 'true' ), home_url() ) );
				exit;
			}
			if ( is_mcp_inactive_user( $user_id ) ) {
				wp_safe_redirect( add_query_arg( array( 'inactive' => 'true' ), home_url() ) );
				exit;
			}
			if ( get_option( 'memberchimp_disable_public_profiles' ) == 'yes' && ! isset( $_GET[ 'login' ] ) ) {
				wp_safe_redirect( add_query_arg( array( 'login' => 'true' ) ) );
				exit;
			}
		break;
	}

	$tab = get_query_var( 'mcp_tab' );

	// Strip the view endpoint.
	if ( $tab === 'view' ) {
		wp_safe_redirect( memberchimp_get_profile_url( $user ) );
		exit;
	}

	// Can not edit.
	if ( $tab === 'edit' && ! mcp_user_can_edit_profile( $user_id ) ) {
		wp_safe_redirect( memberchimp_get_profile_url( $user ) );
		exit;
	}

	// Let's check for disabled endpoints.
	if ( 'no' === get_option( 'memberchimp_' . $tab . '_tab' ) ) {
		wp_safe_redirect( memberchimp_get_profile_url( $user ) );
		exit;
	}
}

/**
 * Get a user profile URL.
 */
function memberchimp_get_profile_url( $user = '', $ajax = false ) {
	global $the_user;

	// When profile is not setup, fallback to author posts url.
	if ( mcp_get_page_id( 'profile' ) <= 0 ) {
		return apply_filters( 'memberchimp_get_profile_url_fallback', get_author_posts_url( is_numeric( $user ) ? $user : username_exists( $user ) ) );
	}

	// If empty, assume we are in global.
	if ( empty( $user ) ) {
		$user = $the_user->get( 'user_login' );
	}

	// Get user login by ID.
	if ( is_numeric( $user ) ) {
		$check_user_id = get_userdata( $user );
		if ( $check_user_id ) {
			$the_user 	= mcp_get_user( $user );
			$user 		= $the_user->user_login;
		}
	}

	$permalink = mcp_get_page_permalink( 'profile' );

	if ( $ajax ) {
		$user = '<span class="mcp-ajax">' . $user . '</span>';
	}

	return apply_filters( 'memberchimp_get_profile_url', mcp_get_endpoint_url( $user, '', $permalink ), $user, $the_user );
}

/**
 * Get the current logged in user profile URL.
 */
function mcp_get_current_user_profile() {
	global $logged_user;

	if ( ! is_user_logged_in() ) {
		return;
	}

	// Get logged user object if it's not yet in memory.
	if ( ! isset( $logged_user->ID ) ) {
		$logged_user = mcp_get_user( get_current_user_id() );
	}

	$user 	   = $logged_user->get( 'user_login' );
	$permalink = mcp_get_page_permalink( 'profile' );

	return apply_filters( 'mcp_get_current_user_profile', mcp_get_endpoint_url( $user, '', $permalink ), $user, $logged_user );
}

/**
 * Get an endpoint for the current logged in user.
 */
function mcp_get_current_user_endpoint( $endpoint ) {
	// Ignore the default 'view'
	if ( $endpoint == 'view' ) {
		$endpoint = null;
	}

	return mcp_get_endpoint_url( $endpoint, '', mcp_get_current_user_profile() );	
}

/**
 * Get profile tabs.
 */
function mcp_get_profile_menu() {
	$array = array(
		'view'			=> array(
			'label'		=> esc_html__( 'Profile', 'memberchimp' ),
			'icon'		=> 'mcp_get_small_avatar',
		),
		'account'		=> array(
			'label'		=> esc_html__( 'Account &amp; Settings', 'memberchimp' ),
			'icon'		=> 'settings',
			'url'		=> mcp_get_page_permalink( 'account' )
		),
	);

	// When user is not logged in.
	if ( ! is_user_logged_in() ) {
		$array = array(
			'register'	=> array(
				'label'		=> esc_html__( 'Register', 'memberchimp' ),
				'icon'		=> 'user-plus',
				'url'		=> mcp_get_page_permalink( 'register' )
			),
			'login'		=> array(
				'label'		=> esc_html__( 'Login', 'memberchimp' ),
				'icon'		=> 'log-in',
				'url'		=> mcp_get_page_permalink( 'login' )
			),
		);

		return apply_filters( 'mcp_get_profile_menu_noauth', $array );
	}

	return apply_filters( 'mcp_get_profile_menu', $array );
}

/**
 * Add user avatar to profile menu.
 */
function mcp_get_small_avatar() {
	return get_avatar( get_current_user_id(), 48 );
}

/**
 * Add log-out link to the end of profile links.
 */
add_filter( 'mcp_get_profile_menu', 'mcp_add_logout_link', 9999 );
function mcp_add_logout_link( $array ) {
	$array[ 'logout' ] = array(
		'label'		=> esc_html__( 'Log out', 'memberchimp' ),
		'icon'		=> 'log-out',
		'url'		=> mcp_logout_url( mcp_get_current_url() ),
	);

	return $array;
}

/**
 * Display profile tab items.
 */
function mcp_get_profile_tabs() {
	$array = array(
		'view'			=> array(
			'label'		=> esc_html__( 'About', 'memberchimp' ),
		),
		'posts'			=> array(
			'label'		=> esc_html__( 'Posts', 'memberchimp' ),
			'count'		=> true,
			'count_cb'  => 'mcp_get_user_posts_count',
		),
		'comments'		=> array(
			'label'		=> esc_html__( 'Comments', 'memberchimp' ),
			'count'		=> true,
			'count_cb'  => 'mcp_get_user_comments_count',
		),
	);

	// Unset 'posts' tab if its turned off.
	if ( 'no' === get_option( 'memberchimp_posts_tab' ) ) {
		unset( $array[ 'posts' ] );
	}

	// Unset 'comments' tab if its turned off.
	if ( 'no' === get_option( 'memberchimp_comments_tab' ) ) {
		unset( $array[ 'comments' ] );
	}

	return apply_filters( 'mcp_get_profile_tabs', $array );
}

/**
 * Get profile default endpoint.
 */
function mcp_get_profile_default_endpoint() {
	return apply_filters( 'mcp_get_profile_default_endpoint', 'view' );
}

/**
 * Get profile endpoint URL.
 */
function mcp_get_profile_endpoint_url( $endpoint, $user = null ) {
	// Ignore the default 'view'
	if ( $endpoint == 'view' ) {
		$endpoint = null;
	}

	return mcp_get_endpoint_url( $endpoint, '', memberchimp_get_profile_url( $user ) );
}

/**
 * Gets the link to the users edit profile page.
 */
function memberchimp_get_edit_user_link( $user_id = 0 ) {
	$link = mcp_get_page_id( 'profile' ) > 0 ? mcp_get_endpoint_url( 'edit', '', memberchimp_get_profile_url( $user_id ) ) : get_edit_user_link( $user_id );
	return apply_filters( 'memberchimp_get_edit_user_link', $link );
}

/**
 * Get profile endpoint form.
 */
function mcp_get_profile_endpoint_form() {
	$endpoint = get_query_var( 'mcp_tab' );

	if ( $endpoint && ! in_array( $endpoint, array( 'edit', 'view' ) ) ) {
		return get_option( 'memberchimp_profile_' . mcp_sanitize_title( $endpoint ) . '_form' );
	}

	return apply_filters( 'memberchimp_profile_form_id', get_option( 'memberchimp_profile_form', '' ) );
}

/**
 * This loads a different form per user role.
 */
function memberchimp_profile_form_id( $form_id = '' ) {
	global $the_user;
	$form = ! empty( $the_user->ID ) ? absint( get_option( 'memberchimp_profile_form_' . esc_attr( $the_user->get_role() ) ) ) : '';

	$updated_id = $form > 0 ? $form : $form_id;

	if ( $updated_id == $form_id ) {
		$is_forced = get_post_meta( $form_id, 'force_role', true );
		$has_role = get_post_meta( $form_id, 'role', true );
		
		if ( $is_forced == 'yes' && $has_role != $the_user->get_role() ) {
			return 0;
		}
	}

	return $updated_id;
}
add_filter( 'memberchimp_profile_form_id', 'memberchimp_profile_form_id', 1 );

/**
 * Get profile menu items.
 */
function mcp_get_profile_menu_items( $in_tabs = false ) {
	$endpoints = $in_tabs ? mcp_get_profile_tabs() : mcp_get_profile_menu();

	// Fail if endpoints are empty.
	if ( empty( $endpoints ) ) {
		return;
	}

	$default = mcp_get_profile_default_endpoint();

	// Remove missing endpoints.
	foreach ( $endpoints as $endpoint_id => $endpoint ) {
		if ( empty( $endpoint ) ) {
			unset( $items[ $endpoint_id ] );
		}
		$items[ $endpoint_id ] = $endpoint;
	}

	// Make sure that default endpoint comes on top.
	if ( isset( $items[ $default ] ) ) {
		$default_endpoint = $items[ $default ];
		unset( $items[ $default ] );
		$items = array_merge( array( $default => $default_endpoint ), $items );
	}

	return apply_filters( 'mcp_get_profile_menu_items', $items, $endpoints );
}

/**
 * Get profile menu items classes.
 */
function mcp_get_profile_menu_item_classes( $endpoint, $in_tabs = false ) {
	global $wp, $the_user, $logged_user;

	$classes = array(
		'memberchimp-profile-navigation-link',
		'memberchimp-profile-navigation-link--' . $endpoint,
	);

	$current = get_query_var( 'mcp_tab' ) != '' && get_query_var( 'mcp_tab' ) == $endpoint ? true : false;

	// Default tab.
	if ( ! get_query_var( 'mcp_tab' ) && ( $endpoint === mcp_get_profile_default_endpoint() ) ) {
		if ( is_mcp_profile_page() ) {
			$current = true;
		}
	}

	if ( is_mcp_account_page() && $endpoint == 'account' ) {
		$current = true;
	}

	// When we are editing a profile.
	if ( get_query_var( 'mcp_tab' ) == 'edit' && $endpoint == 'view' ) {
		$current[] = 'is-active';
	}

	// Highlight the correct menu item.
	if ( $current ) {
		if ( ! $in_tabs && get_current_user_id() === $the_user->ID ) {
			$classes[] = 'is-active';
		} else if ( $in_tabs ) {
			$classes[] = 'is-active';
		}
	}

	$classes = apply_filters( 'memberchimp_profile_menu_item_classes', $classes, $endpoint );

	return implode( ' ', array_map( 'sanitize_html_class', $classes ) );
}

/**
 * Display the icon related to an item in profile menu.
 */
function mcp_profile_menu_icon( $data ) {
	if ( ! empty( $data['icon'] ) ) {
		if ( strstr( $data['icon'], 'mcp_' ) ) {
			if ( function_exists( $data['icon'] ) ) {
				echo call_user_func( $data['icon'] );
			}
		} else {
			echo mcp_svg_icon( $data['icon'] );
		}
	}
}

/**
 * Display the count for a menu item.
 */
function mcp_profile_menu_count( $data ) {
	if ( isset( $data['count'], $data['count_cb'] ) && function_exists( $data['count_cb'] ) ) {
	?>
	<span class="mcp-nav-num count count-<?php echo absint( call_user_func( $data['count_cb'] ) ); ?>">
			<?php echo number_format_i18n( call_user_func( $data['count_cb'] ) ); ?>
	</span>
	<?php
	}
}

/**
 * Get profile classes.
 */
function mcp_get_profile_classes( $classes = array() ) {
	global $the_form;

	if ( $the_form->hide_cover === 'yes' ) {
		$classes[] = 'no-cover';
	}

	return apply_filters( 'mcp_get_profile_classes', array_unique( $classes ) );
}