<?php
/**
 * Role Functions.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Get a role.
 */
function mcp_get_role( $role_id = '' ) {
	return new MCP_Role( absint( $role_id ) );
}

/**
 * Add a WordPress role.
 */
function memberchimp_add_role( $role = '', $label = '', $capabilities = '' ) {
	if ( ! $label ) {
		$label = ucfirst( $role );
	}

	$capabilities = array_merge( $capabilities, array( 'read' => true, 'level_0' => true ) );

	add_role( $role, $label, $capabilities );

	// update current plugin roles.
	$current_roles = get_option( 'memberchimp_roles' );

	if ( ! in_array( $role, (array) $current_roles ) ) {
		$current_roles[] = $role;
		update_option( 'memberchimp_roles', $current_roles );
	}
}

/**
 * Get a WordPress role.
 */
function memberchimp_get_role( $role = '' ) {
	global $wp_roles;

	if ( ! class_exists( 'WP_Roles' ) ) {
		return;
	}

	if ( ! isset( $wp_roles ) ) {
		$wp_roles = new WP_Roles();
	}

	return $wp_roles->get_role( $role );
}

/**
 * Assign a specific role to a user.
 */
function memberchimp_set_role( $user_id, $role ) {
	global $current_user;

	if ( ! class_exists( 'WP_User' ) ) {
		return;
	}

	if ( $user_id == 1 ) {
		return;
	}

	if ( $current_user->ID == 1 ) {
		$u = new WP_User( $user_id );
		$u->set_role( $role );
	} else {
		$current_user->set_role( $role );
	}
}

/**
 * Get a list of available WP roles.
 */
function memberchimp_get_roles() {
    global $wp_roles;

	$roles = array();

	if ( ! function_exists( 'get_editable_roles' ) ) {
		require_once ABSPATH . 'wp-admin/includes/user.php';
	}

	$editable_roles = array_reverse( get_editable_roles() );

	foreach ( $editable_roles as $role => $details ) {
		$roles[ esc_attr( $role ) ] = translate_user_role( $details['name'] );
	}

	return apply_filters( 'memberchimp_get_roles', $roles );
}

/**
 * Returns the role friendly title.
 */
function mcp_role( $role ) {
	global $wp_roles;
	
	$roles = memberchimp_get_roles();
	
	if ( isset( $roles[ $role ] ) ) {
		return $roles[ $role ];
	} else {
		return '';
	}
}

/**
 * Get a list of plugin roles only.
 */
function mcp_get_roles() {
	return apply_filters( 'mcp_get_roles', get_option( 'memberchimp_roles' ) );
}

/**
 * Get default role.
 */
function mcp_get_default_role() {
	return apply_filters( 'mcp_get_default_role', get_option( 'memberchimp_default_role', 'member' ) );
}

/**
 * Get default capabilities.
 */
function mcp_get_default_capabilities() {
	$array = array(
		'mcp_edit_profile'		 	=> true,
		'mcp_edit_account'			=> true,
		'mcp_view_profiles'			=> true,
		'mcp_view_memberlist'		=> true,
		'mcp_search_memberlist'		=> true,
		'mcp_log_in'				=> true,
	);

	return apply_filters( 'mcp_get_default_capabilities', $array );
}

/**
 * Get admin capabilities.
 */
function mcp_get_admin_capabilities() {
	$capabilities = array(
		'mcp_view_private'			=> true,
		'mcp_view_private_data'		=> true,
		'mcp_edit_users'			=> true,
		'mcp_delete_users'			=> true,
		'mcp_review_users'			=> true,
		'mcp_view_adminbar'			=> true,
		'mcp_view_wpadmin'			=> true,
		'mcp_settings'				=> true,
		'mcp_addons'				=> true,
		'manage_memberchimp'			=> true,
	);

	$capability_types = mcp_get_post_types();

	foreach ( $capability_types as $capability_type ) {
		$capabilities = $capabilities + array(
			"edit_{$capability_type}" 						=> true,
			"read_{$capability_type}" 						=> true,
			"delete_{$capability_type}" 					=> true,
			"edit_{$capability_type}s" 						=> true,
			"edit_others_{$capability_type}s" 				=> true,
			"publish_{$capability_type}s" 					=> true,
			"read_private_{$capability_type}s" 				=> true,
			"delete_{$capability_type}s" 					=> true,
			"delete_private_{$capability_type}s" 			=> true,
			"delete_published_{$capability_type}s" 			=> true,
			"delete_others_{$capability_type}s" 			=> true,
			"edit_private_{$capability_type}s" 				=> true,
			"edit_published_{$capability_type}s" 			=> true,
		);
	}

	return apply_filters( 'mcp_get_admin_capabilities', $capabilities );
}

/**
 * Get WP admin capabilities.
 */
function mcp_get_wp_admin_capabilities() {
	$array = array(
		'read'                   => true,
		'level_9'                => true,
		'level_8'                => true,
		'level_7'                => true,
		'level_6'                => true,
		'level_5'                => true,
		'level_4'                => true,
		'level_3'                => true,
		'level_2'                => true,
		'level_1'                => true,
		'level_0'                => true,
		'read_private_pages'     => true,
		'read_private_posts'     => true,
		'edit_posts'             => true,
		'edit_pages'             => true,
		'edit_published_posts'   => true,
		'edit_published_pages'   => true,
		'edit_private_pages'     => true,
		'edit_private_posts'     => true,
		'edit_others_posts'      => true,
		'edit_others_pages'      => true,
		'publish_posts'          => true,
		'publish_pages'          => true,
		'delete_posts'           => true,
		'delete_pages'           => true,
		'delete_private_pages'   => true,
		'delete_private_posts'   => true,
		'delete_published_pages' => true,
		'delete_published_posts' => true,
		'delete_others_posts'    => true,
		'delete_others_pages'    => true,
		'manage_categories'      => true,
		'manage_links'           => true,
		'moderate_comments'      => true,
		'upload_files'           => true,
		'export'                 => true,
		'import'                 => true,
		'list_users'             => true,
		'edit_theme_options'     => true,
	);

	return apply_filters( 'mcp_get_wp_admin_capabilities', $array );
}

/**
 * Get capability titles.
 */
function mcp_get_cap_titles() {
	$array = array(
		'mcp_edit_profile'			=> esc_html__( 'Edit profile', 'memberchimp' ),
		'mcp_edit_account'			=> esc_html__( 'Edit "My Account"', 'memberchimp' ),
		'mcp_log_in'					=> esc_html__( 'Login to site', 'memberchimp' ),
		'mcp_view_profiles'			=> esc_html__( 'View profiles', 'memberchimp' ),
		'mcp_view_private'			=> esc_html__( 'View private profiles', 'memberchimp' ),
		'mcp_view_private_data'		=> esc_html__( 'View private info', 'memberchimp' ),
		'mcp_edit_users'				=> esc_html__( 'Edit users', 'memberchimp' ),
		'mcp_review_users'			=> esc_html__( 'Review users', 'memberchimp' ),
		'mcp_delete_users'			=> esc_html__( 'Delete users', 'memberchimp' ),
		'mcp_view_adminbar'			=> esc_html__( 'View admin bar', 'memberchimp' ),
		'mcp_view_wpadmin'			=> esc_html__( 'View wp-admin', 'memberchimp' ),
		'mcp_view_memberlist'		=> esc_html__( 'View member lists', 'memberchimp' ),
		'mcp_search_memberlist'		=> esc_html__( 'Search member lists', 'memberchimp' ),
		'manage_memberchimp'		=> esc_html__( 'Community manager', 'memberchimp' ),
		'mcp_settings'				=> esc_html__( 'Edit community settings', 'memberchimp' ),
		'mcp_addons'					=> esc_html__( 'Manage community add-ons', 'memberchimp' ),
		'publish_mcp_forms'			=> esc_html__( 'Create forms', 'memberchimp' ),
		'publish_mcp_fields'			=> esc_html__( 'Create custom fields', 'memberchimp' ),
		'publish_mcp_roles'			=> esc_html__( 'Create user roles', 'memberchimp' ),
		'publish_mcp_memberlists'	=> esc_html__( 'Create member lists', 'memberchimp' ),
		'edit_mcp_forms'				=> esc_html__( 'Manage forms', 'memberchimp' ),
		'edit_mcp_fields'			=> esc_html__( 'Manage custom fields', 'memberchimp' ),
		'edit_mcp_roles'				=> esc_html__( 'Manage user roles', 'memberchimp' ),
		'edit_mcp_memberlists'		=> esc_html__( 'Manage member lists', 'memberchimp' ),
		'delete_mcp_forms'			=> esc_html__( 'Delete forms', 'memberchimp' ),
		'delete_mcp_fields'			=> esc_html__( 'Delete custom fields', 'memberchimp' ),
		'delete_mcp_roles'			=> esc_html__( 'Delete user roles', 'memberchimp' ),
		'delete_mcp_memberlists'		=> esc_html__( 'Delete member lists', 'memberchimp' ),
	);

	return apply_filters( 'mcp_get_cap_titles', $array );
}

/**
 * Get a nice display title for a given capability.
 */
function mcp_get_cap_title( $cap ) {
	$array = mcp_get_cap_titles();
	$title = isset( $array[ $cap ] ) ? mcp_clean( $array[ $cap ] ) : '';

	return apply_filters( 'mcp_get_cap_title', $title, $cap );
}

/**
 * Create default user roles.
 */
function memberchimp_create_default_roles() {
	global $wp_roles;

	if ( ! class_exists( 'WP_Roles' ) ) {
		return;
	}

	if ( ! isset( $wp_roles ) ) {
		$wp_roles = new WP_Roles();
	}

	foreach( $wp_roles->roles as $key => $data ) {
		$the_role = new MCP_Role();

		$the_role->set( 'post_title', array_key_exists( 'name', $data ) ? $data['name'] : '' );
		$the_role->set( 'post_name', mcp_clean( wp_unslash( $key ) ) );
		$the_role->set( 'meta_input', array(
				'name'		 => $key,
				'is_created' => 1
		) );

		$the_role->insert();
		$the_role->save( $the_role->meta_input );
	}
}

/**
 * Get admin roles.
 */
function memberchimp_get_admin_roles() {

	return apply_filters( 'memberchimp_get_admin_roles', array( 'administrator', 'community_manager' ) );
}