<?php
/**
 * User Hooks.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Hooks after a valid login - Member Chimp authentication.
 */
add_action( 'wp_authenticate_user', 'memberchimp_authenticate_user_login', 10, 2 );

// User update.
add_action( 'memberchimp_created_user', 'memberchimp_update_user_details', 10, 3 );
add_action( 'memberchimp_account_update', 'memberchimp_update_user_details', 10, 3 );
add_action( 'memberchimp_profile_update', 'memberchimp_update_user_details', 10, 3 );

// Validate usernames.
add_filter( 'validate_username', 'mcp_validate_username', 100, 2 );

// Fired when user is deleted.
add_action( 'mcp_pre_delete_user', 'mcp_delete_user_files', 10, 2 );

/**
 * Authenticate user login and check their account status.
 */
function memberchimp_authenticate_user_login( $user, $password ) {

	// Check that user capability does not allow login.
	if ( ! user_can( $user->ID, 'mcp_log_in' ) && ! user_can( $user->ID, 'manage_options' ) ) {
		return new WP_Error( 'mcp-login-disabled', apply_filters( 'memberchimp_login_disabled_message', esc_html__( 'You do not have enough permissions to log in.', 'memberchimp' ) ) );
	}

	$unconfirmed_email = get_user_meta( $user->ID, '_mcp_unconfirmed_email', true );

	if ( $unconfirmed_email ) {
		return new WP_Error( 'unconfirmed_email', apply_filters( 'memberchimp_unconfirmed_email_message', esc_html__( 'Your account email is unconfirmed.', 'memberchimp' ) ) );
	}

	$pending = get_user_meta( $user->ID, '_mcp_pending', true );

	if ( $pending ) {
		return new WP_Error( 'pending_account', apply_filters( 'memberchimp_pending_account_message', esc_html__( 'Your account is under review. You will be emailed when your account is ready.', 'memberchimp' ) ) );
	}

	return $user;
}

/**
 * Update user details when they register.
 */
function memberchimp_update_user_details( $user_id, $user, $args ) {

	$user->update( $args );

}

/**
 * Only allow alphanumeric and underscores for username.
 */
function mcp_validate_username( $valid, $user ) {
	if ( ! preg_match( '/^[A-Za-z0-9_.]+$/', $user ) ) {
		return false;
	}

	$sanitized 	= sanitize_user( $user, true );
	$valid 		= ( $sanitized == $user );

	return $valid;
}

/**
 * Delete user uploaded files.
 */
function mcp_delete_user_files( $user_id, $user = null ) {

	// Delete all user uploads.
	mcp_delete_user_avatar( $user_id );
	mcp_delete_user_cover( $user_id );
	mcp_delete_user_uploads( $user_id );
}

/**
 * Deletes user avatar.
 */
function mcp_delete_user_avatar( $user_id, $force = false ) {
	return mcp_delete_user_photo( 'avatar', $user_id, $force );
}

/**
 * Deletes user cover.
 */
function mcp_delete_user_cover( $user_id, $force = false ) {
	return mcp_delete_user_photo( 'cover', $user_id, $force );
}

/**
 * A wrapper function to delete a user's photo.
 */
function mcp_delete_user_photo( $name = 'avatar', $user_id, $force = false ) {
	$photo = get_user_meta( $user_id, "_mcp_profile_{$name}", true );

	do_action( "mcp_pre_delete_user_{$name}", $user_id, $photo, $force );

	if ( $photo && ( apply_filters( "mcp_delete_previous_user_{$name}", true ) || $force ) ) {
		$file_path = mcp_plugin_uploads_path( $name ) . '/' . $photo;

		wp_delete_file( $file_path );

		$sizes = call_user_func( "mcp_get_{$name}_sizes" );

		foreach( ( array ) $sizes as $width ) {
			$height = $width;
			if ( $name == 'cover' ) {
				$height = floor( $width / 2.70 );
			}
			$file_path = mcp_plugin_uploads_path( $name ) . '/' . str_replace( '.jpg', "_{$width}x{$height}.jpg", $photo );

			wp_delete_file( $file_path );
		}

		delete_user_meta( $user_id, "_mcp_profile_{$name}" );

		do_action( "mcp_delete_user_{$name}", $user_id, $photo, $force );
	}
}

/**
 * Delete user uploads aside from avatar and header.
 */
function mcp_delete_user_uploads( $user_id = 0 ) {
	$files = get_user_meta( $user_id, '_mcp_files', true );

	if ( ! empty( $files ) ) {
		foreach( apply_filters( 'mcp_delete_user_files', $files ) as $file ) {
			foreach( mcp_supported_upload_types() as $type ) {
				$file_path = mcp_plugin_uploads_path( $type ) . '/' . $file;
				wp_delete_file( $file_path );
				unset( $files[ $file ] );
			}
		}
		delete_user_meta( $user_id, '_mcp_files' );
	}

	// Fired once user uploads are erased.
	do_action( 'mcp_deleted_user_uploads', $user_id );
}